@@ -147,7 +147,9 @@ dump_debug(void *buf, size_t count) {
 	if (!debug_fd)
 		debug_fd = open(debug_file, O_CREAT | O_WRONLY | O_TRUNC, S_IRWXO);
 
-	write(debug_fd, buf, count);
+	if (write(debug_fd, buf, count) == -1)
+		fprintf(stderr, "Error writing to debug file: %s\n",
+			strerror(errno));
 }
 
 static volatile void *
@@ -324,7 +326,7 @@ db_shutdown(int sig) {
 	printf("Shutting down...\n");
 }
 
-static void
+static void __attribute__((noreturn))
 die(int reason) {
 	int i = 0;
 
@@ -549,7 +551,7 @@ int main(int argc, char* argv[]) {
 	 */
 	if (!clear_waits) {
 		int dh_handle;
-		drm_fd = drm_open_any();
+		drm_fd = drm_open_driver(DRIVER_INTEL);
 		bufmgr = drm_intel_bufmgr_gem_init(drm_fd, 4096);
 
 		setup_hw_bits();
