@@ -47,6 +47,7 @@
 #include <sys/syscall.h>
 #include <sys/utsname.h>
 #include <termios.h>
+#include <assert.h>
 
 #include "drmtest.h"
 #include "i915_drm.h"
@@ -58,12 +59,13 @@
 #include "intel_reg.h"
 #include "ioctl_wrappers.h"
 #include "igt_kms.h"
+#include "igt_pm.h"
 
 /**
  * SECTION:igt_aux
  * @short_description: Auxiliary libraries and support functions
- * @title: i-g-t aux
- * @include: igt_aux.h
+ * @title: aux
+ * @include: igt.h
  *
  * This library provides various auxiliary helper functions that don't really
  * fit into any other topic.
@@ -78,7 +80,7 @@ static void __attribute__((noreturn)) signal_helper_process(pid_t pid)
 	/* Interrupt the parent process at 500Hz, just to be annoying */
 	while (1) {
 		usleep(1000 * 1000 / 500);
-		if (kill(pid, SIGUSR1)) /* Parent has died, so must we. */
+		if (kill(pid, SIGCONT)) /* Parent has died, so must we. */
 			exit(0);
 	}
 }
@@ -92,7 +94,7 @@ static void sig_handler(int i)
  * igt_fork_signal_helper:
  *
  * Fork a child process using #igt_fork_helper to interrupt the parent process
- * with a SIGUSR1 signal at regular quick intervals. The corresponding dummy
+ * with a SIGCONT signal at regular quick intervals. The corresponding dummy
  * signal handler is installed in the parent process.
  *
  * This is useful to exercise ioctl error paths, at least where those can be
@@ -107,10 +109,23 @@ void igt_fork_signal_helper(void)
 	if (igt_only_list_subtests())
 		return;
 
-	signal(SIGUSR1, sig_handler);
+	/* We pick SIGCONT as it is a "safe" signal - if we send SIGCONT to
+	 * an unexpecting process it spuriously wakes up and does nothing.
+	 * Most other signals (e.g. SIGUSR1) cause the process to die if they
+	 * are not handled. This is an issue in case the sighandler is not
+	 * inherited correctly (or if there is a race in the inheritance
+	 * and we send the signal at exactly the wrong time).
+	 */
+	signal(SIGCONT, sig_handler);
+	setpgrp(); /* define a new process group for the tests */
 
 	igt_fork_helper(&signal_helper) {
-		signal_helper_process(getppid());
+		setpgrp(); /* Escape from the test process group */
+
+		/* Pass along the test process group identifier,
+		 * negative pid => send signal to everyone in the group.
+		 */
+		signal_helper_process(-getppid());
 	}
 }
 
@@ -230,7 +245,7 @@ static void igt_interactive_info(const char *format, ...)
 {
 	va_list args;
 
-	if (!isatty(STDERR_FILENO))
+	if (!isatty(STDERR_FILENO) || __igt_plain_output)
 		return;
 
 	if (igt_log_level > IGT_LOG_INFO)
@@ -249,7 +264,7 @@ static void igt_interactive_info(const char *format, ...)
  * @total: total amount of work
  *
  * This function draws a progress indicator, which is useful for running
- * long-winded tests manually on the console. To avoid spamming logfiles in
+ * long-winded tests manually on the console. To avoid spamming log files in
  * automated runs the progress indicator is suppressed when not running on a
  * terminal.
  */
@@ -356,6 +371,9 @@ void igt_system_suspend_autoresume(void)
 	 * seems to fare better. We need to investigate what's going on. */
 	igt_skip_on_simulation();
 
+	/* skip if system doesn't support suspend-to-mem */
+	igt_skip_on(system("rtcwake -n -s 30 -m mem") != 0);
+
 	ret = system("rtcwake -s 30 -m mem");
 	igt_assert_f(ret == 0,
 		     "This failure means that something is wrong with the "
@@ -383,6 +401,9 @@ void igt_system_hibernate_autoresume(void)
 	 * seems to fare better. We need to investigate what's going on. */
 	igt_skip_on_simulation();
 
+	/* skip if system doesn't support suspend-to-disk */
+	igt_skip_on(system("rtcwake -n -s 90 -m disk") != 0);
+
 	/* The timeout might need to be adjusted if hibernation takes too long
 	 * or if we have to wait excessively long before resume
 	 */
@@ -469,7 +490,7 @@ void igt_debug_wait_for_keypress(const char *var)
  * When not connected to a terminal interactive_debug is ignored
  * and execution immediately continues. For this reason by default this function
  * returns true. It returns false only when N/n is pressed indicating the
- * user ins't seeing what was expected.
+ * user isn't seeing what was expected.
  *
  * Force test fail when N/n is pressed.
  */
@@ -524,19 +545,7 @@ bool igt_setup_runtime_pm(void)
 	if (pm_status_fd >= 0)
 		return true;
 
-	/* The Audio driver can get runtime PM references, so we need to make
-	 * sure its runtime PM is enabled, so it can release the refs and
-	 * actually enable us to runtime suspend. */
-	fd = open("/sys/module/snd_hda_intel/parameters/power_save", O_WRONLY);
-	if (fd >= 0) {
-		igt_assert(write(fd, "1\n", 2) == 2);
-		close(fd);
-	}
-	fd = open("/sys/bus/pci/devices/0000:00:03.0/power/control", O_WRONLY);
-	if (fd >= 0) {
-		igt_assert(write(fd, "auto\n", 5) == 5);
-		close(fd);
-	}
+	igt_pm_enable_audio_runtime_pm();
 
 	/* Our implementation uses autosuspend. Try to set it to 0ms so the test
 	 * suite goes faster and we have a higher probability of triggering race
@@ -767,8 +776,16 @@ static void igt_module_param_exit_handler(int sig)
 
 		fd = open(file_path, O_RDWR);
 		if (fd >= 0) {
-			write(fd, data->original_value,
-			      strlen(data->original_value));
+			int size = strlen (data->original_value);
+
+			if (size != write(fd, data->original_value, size)) {
+				const char msg[] = "WARNING: Module parameters "
+					"may not have been reset to their "
+					"original values\n";
+				assert(write(STDERR_FILENO, msg, sizeof(msg))
+				       == sizeof(msg));
+			}
+
 			close(fd);
 		}
 	}
@@ -827,7 +844,7 @@ static void igt_save_module_param(const char *name, const char *file_path)
  *
  * This function sets the desired value for the given i915.ko parameter. It also
  * takes care of saving and restoring the values that were already set before
- * the test was run by calling igt_save_module_param().
+ * the test was run.
  *
  * Please consider using igt_set_module_param_int() for the integer and bool
  * parameters.
