@@ -86,7 +86,9 @@ uint64_t intel_get_avail_ram_mb(void);
 uint64_t intel_get_total_ram_mb(void);
 uint64_t intel_get_total_swap_mb(void);
 
-void intel_require_memory(uint32_t count, uint64_t size, unsigned mode);
+int __intel_check_memory(uint64_t count, uint64_t size, unsigned mode,
+			 uint64_t *out_required, uint64_t *out_total);
+void intel_require_memory(uint64_t count, uint64_t size, unsigned mode);
 #define CHECK_RAM 0x1
 #define CHECK_SWAP 0x2
 
