@@ -192,6 +192,55 @@ intel_get_total_swap_mb(void)
 	return retval / (1024*1024);
 }
 
+static uint64_t vfs_file_max(void)
+{
+	static long long unsigned max;
+	if (max == 0) {
+		FILE *file = fopen("/proc/sys/fs/file-max", "r");
+		max = 80000;
+		if (file) {
+			igt_assert(fscanf(file, "%llu", &max) == 1);
+			fclose(file);
+		}
+	}
+	return max;
+}
+
+int __intel_check_memory(uint64_t count, uint64_t size, unsigned mode,
+			 uint64_t *out_required, uint64_t *out_total)
+{
+/* rough estimate of how many bytes the kernel requires to track each object */
+#define KERNEL_BO_OVERHEAD 512
+	uint64_t required, total;
+
+	required = count;
+	required *= size + KERNEL_BO_OVERHEAD;
+	required = ALIGN(required, 4096);
+
+	igt_debug("Checking %'llu surfaces of size %'llu bytes (total %'llu) against %s%s\n",
+		  (long long)count, (long long)size, (long long)required,
+		  mode & (CHECK_RAM | CHECK_SWAP) ? "RAM" : "",
+		  mode & CHECK_SWAP ? " + swap": "");
+
+	total = 0;
+	if (mode & (CHECK_RAM | CHECK_SWAP))
+		total += intel_get_avail_ram_mb();
+	if (mode & CHECK_SWAP)
+		total += intel_get_total_swap_mb();
+	total *= 1024 * 1024;
+
+	if (out_required)
+		*out_required = required;
+
+	if (out_total)
+		*out_total = total;
+
+	if (count > vfs_file_max())
+		return false;
+
+	return required < total;
+}
+
 /**
  * intel_require_memory:
  * @count: number of surfaces that will be created
@@ -215,33 +264,19 @@ intel_get_total_swap_mb(void)
  * assumption that any test that needs to check for memory requirements is a
  * thrashing test unsuitable for slow simulated systems.
  */
-void intel_require_memory(uint32_t count, uint64_t size, unsigned mode)
+void intel_require_memory(uint64_t count, uint64_t size, unsigned mode)
 {
-/* rough estimate of how many bytes the kernel requires to track each object */
-#define KERNEL_BO_OVERHEAD 512
 	uint64_t required, total;
 
-	required = count;
-	required *= size + KERNEL_BO_OVERHEAD;
-	required = ALIGN(required, 4096);
-
-	igt_debug("Checking %'u surfaces of size %'llu bytes (total %'llu) against %s%s\n",
-		  count, (long long)size, (long long)required,
-		  mode & (CHECK_RAM | CHECK_SWAP) ? "RAM" : "",
-		  mode & CHECK_SWAP ? " + swap": "");
-
-	total = 0;
-	if (mode & (CHECK_RAM | CHECK_SWAP))
-		total += intel_get_avail_ram_mb();
-	if (mode & CHECK_SWAP)
-		total += intel_get_total_swap_mb();
-	total *= 1024 * 1024;
-
-	igt_skip_on_f(total <= required,
-		      "Estimated that we need %'llu bytes for the test, but only have %'llu bytes available (%s%s)\n",
-		      (long long)required, (long long)total,
+	igt_skip_on_f(!__intel_check_memory(count, size, mode,
+					    &required, &total),
+		      "Estimated that we need %'llu objects and %'llu MiB for the test, but only have %'llu MiB available (%s%s) and a maximum of %'llu objects\n",
+		      (long long)count,
+		      (long long)((required + ((1<<20) - 1)) >> 20),
+		      (long long)(total >> 20),
 		      mode & (CHECK_RAM | CHECK_SWAP) ? "RAM" : "",
-		      mode & CHECK_SWAP ? " + swap": "");
+		      mode & CHECK_SWAP ? " + swap": "",
+		      (long long)vfs_file_max());
 
 	igt_skip_on_simulation();
 }
