@@ -199,9 +199,10 @@ void gem_set_caching(int fd, uint32_t handle, uint32_t caching)
 	memset(&arg, 0, sizeof(arg));
 	arg.handle = handle;
 	arg.caching = caching;
-	ret = ioctl(fd, LOCAL_DRM_IOCTL_I915_GEM_SET_CACHEING, &arg);
 
+	ret = drmIoctl(fd, LOCAL_DRM_IOCTL_I915_GEM_SET_CACHEING, &arg);
 	igt_assert(ret == 0 || (errno == ENOTTY || errno == EINVAL));
+
 	igt_require(ret == 0);
 	errno = 0;
 }
@@ -290,6 +291,8 @@ void gem_close(int fd, uint32_t handle)
 {
 	struct drm_gem_close close_bo;
 
+	igt_assert_neq(handle, 0);
+
 	memset(&close_bo, 0, sizeof(close_bo));
 	close_bo.handle = handle;
 	do_ioctl(fd, DRM_IOCTL_GEM_CLOSE, &close_bo);
@@ -366,6 +369,36 @@ void gem_set_domain(int fd, uint32_t handle,
 	do_ioctl(fd, DRM_IOCTL_I915_GEM_SET_DOMAIN, &set_domain);
 }
 
+/**
+ * __gem_wait:
+ * @fd: open i915 drm file descriptor
+ * @handle: gem buffer object handle
+ * @timeout_ns: [in] time to wait, [out] remaining time (in nanoseconds)
+ *
+ * This functions waits for outstanding rendering to complete, upto
+ * the timeout_ns. If no timeout_ns is provided, the wait is indefinite and
+ * only returns upon an error or when the rendering is complete.
+ */
+int gem_wait(int fd, uint32_t handle, int64_t *timeout_ns)
+{
+	struct drm_i915_gem_wait wait;
+	int ret;
+
+	memset(&wait, 0, sizeof(wait));
+	wait.bo_handle = handle;
+	wait.timeout_ns = timeout_ns ? *timeout_ns : -1;
+	wait.flags = 0;
+
+	ret = 0;
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_WAIT, &wait))
+		ret = -errno;
+
+	if (timeout_ns)
+		*timeout_ns = wait.timeout_ns;
+
+	return ret;
+}
+
 /**
  * gem_sync:
  * @fd: open i915 drm file descriptor
@@ -375,20 +408,86 @@ void gem_set_domain(int fd, uint32_t handle,
  */
 void gem_sync(int fd, uint32_t handle)
 {
-	struct drm_i915_gem_wait wait;
+	if (gem_wait(fd, handle, NULL))
+		gem_set_domain(fd, handle,
+			       I915_GEM_DOMAIN_GTT,
+			       I915_GEM_DOMAIN_GTT);
+	errno = 0;
+}
 
-	memset(&wait, 0, sizeof(wait));
-	wait.bo_handle = handle;
-	wait.timeout_ns =-1;
-	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_WAIT, &wait) == 0) {
-		errno = 0;
-		return;
+
+bool gem_create__has_stolen_support(int fd)
+{
+	static int has_stolen_support = -1;
+	struct drm_i915_getparam gp;
+	int val = -1;
+
+	if (has_stolen_support < 0) {
+		memset(&gp, 0, sizeof(gp));
+		gp.param = 36; /* CREATE_VERSION */
+		gp.value = &val;
+
+		/* Do we have the extended gem_create_ioctl? */
+		ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp);
+		has_stolen_support = val >= 2;
 	}
 
-	gem_set_domain(fd, handle,
-		       I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+	return has_stolen_support;
 }
 
+struct local_i915_gem_create_v2 {
+	uint64_t size;
+	uint32_t handle;
+	uint32_t pad;
+#define I915_CREATE_PLACEMENT_STOLEN (1<<0)
+	uint32_t flags;
+};
+
+#define LOCAL_IOCTL_I915_GEM_CREATE       DRM_IOWR(DRM_COMMAND_BASE + DRM_I915_GEM_CREATE, struct local_i915_gem_create_v2)
+uint32_t __gem_create_stolen(int fd, uint64_t size)
+{
+	struct local_i915_gem_create_v2 create;
+	int ret;
+
+	memset(&create, 0, sizeof(create));
+	create.handle = 0;
+	create.size = size;
+	create.flags = I915_CREATE_PLACEMENT_STOLEN;
+	ret = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_CREATE, &create);
+
+	if (ret < 0)
+		return 0;
+
+	errno = 0;
+	return create.handle;
+}
+
+/**
+ * gem_create_stolen:
+ * @fd: open i915 drm file descriptor
+ * @size: desired size of the buffer
+ *
+ * This wraps the new GEM_CREATE ioctl, which allocates a new gem buffer
+ * object of @size and placement in stolen memory region.
+ *
+ * Returns: The file-private handle of the created buffer object
+ */
+
+uint32_t gem_create_stolen(int fd, uint64_t size)
+{
+	struct local_i915_gem_create_v2 create;
+
+	memset(&create, 0, sizeof(create));
+	create.handle = 0;
+	create.size = size;
+	create.flags = I915_CREATE_PLACEMENT_STOLEN;
+	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_CREATE, &create);
+	igt_assert(create.handle);
+
+	return create.handle;
+}
+
+
 uint32_t __gem_create(int fd, int size)
 {
 	struct drm_i915_gem_create create;
@@ -429,6 +528,22 @@ uint32_t gem_create(int fd, uint64_t size)
 	return create.handle;
 }
 
+/**
+ * __gem_execbuf:
+ * @fd: open i915 drm file descriptor
+ * @execbuf: execbuffer data structure
+ *
+ * This wraps the EXECBUFFER2 ioctl, which submits a batchbuffer for the gpu to
+ * run. This is allowed to fail, with -errno returned.
+ */
+int __gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
+{
+	int err = 0;
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, execbuf))
+		err = -errno;
+	return err;
+}
+
 /**
  * gem_execbuf:
  * @fd: open i915 drm file descriptor
@@ -439,15 +554,12 @@ uint32_t gem_create(int fd, uint64_t size)
  */
 void gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
 {
-	int result;
-
-	result = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, execbuf);
-	igt_assert(result == 0);
+	igt_assert_eq(__gem_execbuf(fd, execbuf), 0);
 	errno = 0;
 }
 
 /**
- * gem_mmap__gtt:
+ * __gem_mmap__gtt:
  * @fd: open i915 drm file descriptor
  * @handle: gem buffer object handle
  * @size: size of the gem buffer
@@ -456,9 +568,9 @@ void gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
  * This functions wraps up procedure to establish a memory mapping through the
  * GTT.
  *
- * Returns: A pointer to the created memory mapping.
+ * Returns: A pointer to the created memory mapping, NULL on failure.
  */
-void *gem_mmap__gtt(int fd, uint32_t handle, uint64_t size, unsigned prot)
+void *__gem_mmap__gtt(int fd, uint32_t handle, uint64_t size, unsigned prot)
 {
 	struct drm_i915_gem_mmap_gtt mmap_arg;
 	void *ptr;
@@ -477,6 +589,24 @@ void *gem_mmap__gtt(int fd, uint32_t handle, uint64_t size, unsigned prot)
 	return ptr;
 }
 
+/**
+ * gem_mmap__gtt:
+ * @fd: open i915 drm file descriptor
+ * @handle: gem buffer object handle
+ * @size: size of the gem buffer
+ * @prot: memory protection bits as used by mmap()
+ *
+ * Like __gem_mmap__gtt() except we assert on failure.
+ *
+ * Returns: A pointer to the created memory mapping
+ */
+void *gem_mmap__gtt(int fd, uint32_t handle, uint64_t size, unsigned prot)
+{
+	void *ptr = __gem_mmap__gtt(fd, handle, size, prot);
+	igt_assert(ptr);
+	return ptr;
+}
+
 struct local_i915_gem_mmap_v2 {
 	uint32_t handle;
 	uint32_t pad;
@@ -523,7 +653,7 @@ bool gem_mmap__has_wc(int fd)
 }
 
 /**
- * gem_mmap__wc:
+ * __gem_mmap__wc:
  * @fd: open i915 drm file descriptor
  * @handle: gem buffer object handle
  * @offset: offset in the gem buffer of the mmap arena
@@ -535,9 +665,9 @@ bool gem_mmap__has_wc(int fd)
  * bypassing the GTT system agent (i.e. there is no automatic tiling of
  * the mmapping through the fence registers).
  *
- * Returns: A pointer to the created memory mapping.
+ * Returns: A pointer to the created memory mapping, NULL on failure.
  */
-void *gem_mmap__wc(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
+void *__gem_mmap__wc(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
 {
 	struct local_i915_gem_mmap_v2 arg;
 
@@ -559,7 +689,26 @@ void *gem_mmap__wc(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsi
 }
 
 /**
- * gem_mmap__cpu:
+ * gem_mmap__wc:
+ * @fd: open i915 drm file descriptor
+ * @handle: gem buffer object handle
+ * @offset: offset in the gem buffer of the mmap arena
+ * @size: size of the mmap arena
+ * @prot: memory protection bits as used by mmap()
+ *
+ * Like __gem_mmap__wc() except we assert on failure.
+ *
+ * Returns: A pointer to the created memory mapping
+ */
+void *gem_mmap__wc(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
+{
+	void *ptr = __gem_mmap__wc(fd, handle, offset, size, prot);
+	igt_assert(ptr);
+	return ptr;
+}
+
+/**
+ * __gem_mmap__cpu:
  * @fd: open i915 drm file descriptor
  * @handle: gem buffer object handle
  * @offset: offset in the gem buffer of the mmap arena
@@ -569,9 +718,9 @@ void *gem_mmap__wc(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsi
  * This functions wraps up procedure to establish a memory mapping through
  * direct cpu access, bypassing the gpu completely.
  *
- * Returns: A pointer to the created memory mapping.
+ * Returns: A pointer to the created memory mapping, NULL on failure.
  */
-void *gem_mmap__cpu(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
+void *__gem_mmap__cpu(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
 {
 	struct drm_i915_gem_mmap mmap_arg;
 
@@ -586,6 +735,25 @@ void *gem_mmap__cpu(int fd, uint32_t handle, uint64_t offset, uint64_t size, uns
 	return (void *)(uintptr_t)mmap_arg.addr_ptr;
 }
 
+/**
+ * gem_mmap__cpu:
+ * @fd: open i915 drm file descriptor
+ * @handle: gem buffer object handle
+ * @offset: offset in the gem buffer of the mmap arena
+ * @size: size of the mmap arena
+ * @prot: memory protection bits as used by mmap()
+ *
+ * Like __gem_mmap__cpu() except we assert on failure.
+ *
+ * Returns: A pointer to the created memory mapping
+ */
+void *gem_mmap__cpu(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
+{
+	void *ptr = __gem_mmap__cpu(fd, handle, offset, size, prot);
+	igt_assert(ptr);
+	return ptr;
+}
+
 /**
  * gem_madvise:
  * @fd: open i915 drm file descriptor
@@ -627,12 +795,14 @@ int gem_madvise(int fd, uint32_t handle, int state)
 uint32_t gem_context_create(int fd)
 {
 	struct drm_i915_gem_context_create create;
-	int ret;
 
 	memset(&create, 0, sizeof(create));
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_CREATE, &create);
-	igt_require(ret == 0 || (errno != ENODEV && errno != EINVAL));
-	igt_assert(ret == 0);
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_CREATE, &create)) {
+		int err = -errno;
+		igt_skip_on(err == -ENODEV || errno == -EINVAL);
+		igt_assert_eq(err, 0);
+	}
+	igt_assert(create.ctx_id != 0);
 	errno = 0;
 
 	return create.ctx_id;
@@ -687,6 +857,16 @@ void gem_context_get_param(int fd, struct local_i915_gem_context_param *p)
 	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, p);
 }
 
+int __gem_context_set_param(int fd, struct local_i915_gem_context_param *p)
+{
+#define LOCAL_I915_GEM_CONTEXT_SETPARAM       0x35
+#define LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM DRM_IOWR (DRM_COMMAND_BASE + LOCAL_I915_GEM_CONTEXT_SETPARAM, struct local_i915_gem_context_param)
+	if (drmIoctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM, p))
+		return -errno;
+
+	errno = 0;
+	return 0;
+}
 /**
  * gem_context_set_param:
  * @fd: open i915 drm file descriptor
@@ -699,9 +879,7 @@ void gem_context_get_param(int fd, struct local_i915_gem_context_param *p)
  */
 void gem_context_set_param(int fd, struct local_i915_gem_context_param *p)
 {
-#define LOCAL_I915_GEM_CONTEXT_SETPARAM       0x35
-#define LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM DRM_IOWR (DRM_COMMAND_BASE + LOCAL_I915_GEM_CONTEXT_SETPARAM, struct local_i915_gem_context_param)
-	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM, p);
+	igt_assert(__gem_context_set_param(fd, p) == 0);
 }
 
 /**
@@ -742,6 +920,47 @@ void gem_context_require_ban_period(int fd)
 	igt_require(has_ban_period);
 }
 
+int __gem_userptr(int fd, void *ptr, int size, int read_only, uint32_t flags, uint32_t *handle)
+{
+	struct local_i915_gem_userptr userptr;
+	int ret;
+
+	memset(&userptr, 0, sizeof(userptr));
+	userptr.user_ptr = (uintptr_t)ptr;
+	userptr.user_size = size;
+	userptr.flags = flags;
+	if (read_only)
+		userptr.flags |= LOCAL_I915_USERPTR_READ_ONLY;
+
+	ret = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr);
+	if (ret)
+		ret = errno;
+	igt_skip_on_f(ret == ENODEV &&
+			(flags & LOCAL_I915_USERPTR_UNSYNCHRONIZED) == 0 &&
+			!read_only,
+			"Skipping, synchronized mappings with no kernel CONFIG_MMU_NOTIFIER?");
+	if (ret == 0)
+		*handle = userptr.handle;
+
+	return ret;
+}
+
+/**
+ * gem_userptr:
+ * @fd: open i915 drm file descriptor
+ * @ptr: userptr pointer to be passed
+ * @size: desired size of the buffer
+ * @read_only: specify whether userptr is opened read only
+ * @flags: other userptr flags
+ * @handle: returned handle for the object
+ *
+ * Returns userptr handle for the GEM object.
+ */
+void gem_userptr(int fd, void *ptr, int size, int read_only, uint32_t flags, uint32_t *handle)
+{
+	igt_assert_eq(__gem_userptr(fd, ptr, size, read_only, flags, handle), 0);
+}
+
 /**
  * gem_sw_finish:
  * @fd: open i915 drm file descriptor
@@ -787,18 +1006,18 @@ bool gem_bo_busy(int fd, uint32_t handle)
 /* feature test helpers */
 
 /**
- * gem_uses_aliasing_ppgtt:
+ * gem_gtt_type:
  * @fd: open i915 drm file descriptor
  *
- * Feature test macro to check whether the kernel internally uses ppgtt to
- * execute batches. The /aliasing/ in the function name is a bit a misnomer,
- * this driver parameter is also true when full ppgtt address spaces are
- * available since for batchbuffer construction only ppgtt or global gtt is
- * relevant.
+ * Feature test macro to check what type of gtt is being used by the kernel:
+ * 0 - global gtt
+ * 1 - aliasing ppgtt
+ * 2 - full ppgtt, limited to 32bit address space
+ * 3 - full ppgtt, 64bit address space
  *
- * Returns: Whether batches are run through ppgtt.
+ * Returns: Type of gtt being used.
  */
-bool gem_uses_aliasing_ppgtt(int fd)
+int gem_gtt_type(int fd)
 {
 	struct drm_i915_getparam gp;
 	int val = 0;
@@ -814,6 +1033,35 @@ bool gem_uses_aliasing_ppgtt(int fd)
 	return val;
 }
 
+/**
+ * gem_uses_ppgtt:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to check whether the kernel internally uses ppgtt to
+ * execute batches. Note that this is also true when we're using full ppgtt.
+ *
+ * Returns: Whether batches are run through ppgtt.
+ */
+bool gem_uses_ppgtt(int fd)
+{
+	return gem_gtt_type(fd) > 0;
+}
+
+/**
+ * gem_uses_full_ppgtt:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to check whether the kernel internally uses full
+ * per-process gtt to execute batches. Note that this is also true when we're
+ * using full 64b ppgtt.
+ *
+ * Returns: Whether batches are run through full ppgtt.
+ */
+bool gem_uses_full_ppgtt(int fd)
+{
+	return gem_gtt_type(fd) > 1;
+}
+
 /**
  * gem_available_fences:
  * @fd: open i915 drm file descriptor
@@ -1035,13 +1283,21 @@ uint64_t gem_aperture_size(int fd)
 	static uint64_t aperture_size = 0;
 
 	if (aperture_size == 0) {
-		struct drm_i915_gem_get_aperture aperture;
+		struct local_i915_gem_context_param p;
 
-		memset(&aperture, 0, sizeof(aperture));
-		aperture.aper_size = 256*1024*1024;
-		do_ioctl(fd, DRM_IOCTL_I915_GEM_GET_APERTURE, &aperture);
+		memset(&p, 0, sizeof(p));
+		p.param = 0x3;
+		if (ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0) {
+			aperture_size = p.value;
+		} else {
+			struct drm_i915_gem_get_aperture aperture;
 
-		aperture_size =  aperture.aper_size;
+			memset(&aperture, 0, sizeof(aperture));
+			aperture.aper_size = 256*1024*1024;
+
+			do_ioctl(fd, DRM_IOCTL_I915_GEM_GET_APERTURE, &aperture);
+			aperture_size =  aperture.aper_size;
+		}
 	}
 
 	return aperture_size;
@@ -1068,6 +1324,54 @@ uint64_t gem_mappable_aperture_size(void)
 	return pci_dev->regions[bar].size;
 }
 
+/**
+ * gem_global_aperture_size:
+ *
+ * Feature test macro to query the kernel for the global gpu aperture size.
+ * This is the area available for the kernel to perform address translations.
+ *
+ * Returns: The mappable gtt address space size.
+ */
+uint64_t gem_global_aperture_size(int fd)
+{
+	struct drm_i915_gem_get_aperture aperture;
+
+	memset(&aperture, 0, sizeof(aperture));
+	aperture.aper_size = 256*1024*1024;
+	do_ioctl(fd, DRM_IOCTL_I915_GEM_GET_APERTURE, &aperture);
+
+	return aperture.aper_size;
+}
+
+#define LOCAL_I915_PARAM_HAS_EXEC_SOFTPIN 37
+/**
+ * gem_has_softpin:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to query whether the softpinning functionality is
+ * supported.
+ *
+ * Returns: Whether softpin support is available
+ */
+bool gem_has_softpin(int fd)
+{
+	static int has_softpin = -1;
+
+	if (has_softpin < 0) {
+		struct drm_i915_getparam gp;
+
+		memset(&gp, 0, sizeof(gp));
+		gp.param = LOCAL_I915_PARAM_HAS_EXEC_SOFTPIN;
+		gp.value = &has_softpin;
+
+		has_softpin = 0;
+		ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp));
+		errno = 0;
+	}
+
+	return has_softpin;
+}
+
 /**
  * gem_require_caching:
  * @fd: open i915 drm file descriptor
@@ -1092,6 +1396,27 @@ void gem_require_caching(int fd)
 	errno = 0;
 }
 
+static int gem_has_ring(int fd, int ring)
+{
+	uint32_t bbe = MI_BATCH_BUFFER_END;
+	struct drm_i915_gem_execbuffer2 execbuf;
+	struct drm_i915_gem_exec_object2 exec;
+	int ret;
+
+	memset(&exec, 0, sizeof(exec));
+	exec.handle = gem_create(fd, 4096);
+	gem_write(fd, exec.handle, 0, &bbe, sizeof(bbe));
+
+	memset(&execbuf, 0, sizeof(execbuf));
+	execbuf.buffers_ptr = (uintptr_t)&exec;
+	execbuf.buffer_count = 1;
+	execbuf.flags = ring;
+	ret = __gem_execbuf(fd, &execbuf);
+	gem_close(fd, exec.handle);
+
+	return ret == 0;
+}
+
 /**
  * gem_require_ring:
  * @fd: open i915 drm file descriptor
@@ -1103,23 +1428,12 @@ void gem_require_caching(int fd)
  */
 void gem_require_ring(int fd, int ring_id)
 {
-	switch (ring_id) {
-	case I915_EXEC_RENDER:
-		return;
-	case I915_EXEC_BLT:
-		igt_require(gem_has_blt(fd));
-		return;
-	case I915_EXEC_BSD:
-		igt_require(gem_has_bsd(fd));
-		return;
-#ifdef I915_EXEC_VEBOX
-	case I915_EXEC_VEBOX:
-		igt_require(gem_has_vebox(fd));
-		return;
-#endif
-	default:
-		igt_assert(0);
-		return;
+	igt_require(gem_has_ring(fd, ring_id));
+
+	/* silly ABI, the kernel thinks everyone who has BSD also has BSD2 */
+	if ((ring_id & ~(3<<13)) == I915_EXEC_BSD) {
+		if (ring_id & (3 << 13))
+			igt_require(gem_has_bsd2(fd));
 	}
 }
 
@@ -1150,6 +1464,31 @@ int prime_handle_to_fd(int fd, uint32_t handle)
 	return args.fd;
 }
 
+/**
+ * prime_handle_to_fd_for_mmap:
+ * @fd: open i915 drm file descriptor
+ * @handle: file-private gem buffer object handle
+ *
+ * Same as prime_handle_to_fd above but with DRM_RDWR capabilities, which can
+ * be useful for writing into the mmap'ed dma-buf file-descriptor.
+ *
+ * Returns: The created dma-buf fd handle or -1 if the ioctl fails.
+ */
+int prime_handle_to_fd_for_mmap(int fd, uint32_t handle)
+{
+	struct drm_prime_handle args;
+
+	memset(&args, 0, sizeof(args));
+	args.handle = handle;
+	args.flags = DRM_CLOEXEC | DRM_RDWR;
+	args.fd = -1;
+
+	if (drmIoctl(fd, DRM_IOCTL_PRIME_HANDLE_TO_FD, &args) != 0)
+		return -1;
+
+	return args.fd;
+}
+
 /**
  * prime_fd_to_handle:
  * @fd: open i915 drm file descriptor
@@ -1196,6 +1535,38 @@ off_t prime_get_size(int dma_buf_fd)
 	return ret;
 }
 
+/**
+ * prime_sync_start
+ * @dma_buf_fd: dma-buf fd handle
+ */
+void prime_sync_start(int dma_buf_fd, bool write)
+{
+	struct local_dma_buf_sync sync_start;
+
+	memset(&sync_start, 0, sizeof(sync_start));
+	sync_start.flags = LOCAL_DMA_BUF_SYNC_START;
+	sync_start.flags |= LOCAL_DMA_BUF_SYNC_READ;
+	if (write)
+		sync_start.flags |= LOCAL_DMA_BUF_SYNC_WRITE;
+	do_ioctl(dma_buf_fd, LOCAL_DMA_BUF_IOCTL_SYNC, &sync_start);
+}
+
+/**
+ * prime_sync_end
+ * @dma_buf_fd: dma-buf fd handle
+ */
+void prime_sync_end(int dma_buf_fd, bool write)
+{
+	struct local_dma_buf_sync sync_end;
+
+	memset(&sync_end, 0, sizeof(sync_end));
+	sync_end.flags = LOCAL_DMA_BUF_SYNC_END;
+	sync_end.flags |= LOCAL_DMA_BUF_SYNC_READ;
+	if (write)
+		sync_end.flags |= LOCAL_DMA_BUF_SYNC_WRITE;
+	do_ioctl(dma_buf_fd, LOCAL_DMA_BUF_IOCTL_SYNC, &sync_end);
+}
+
 /**
  * igt_require_fb_modifiers:
  * @fd: Open DRM file descriptor.
