@@ -1,24 +1,51 @@
 #!/bin/sh
 
+IGT_EXIT_TIMEOUT=78
+IGT_EXIT_SKIP=77
+IGT_EXIT_SUCCESS=0
+IGT_EXIT_INVALID=79
+IGT_EXIT_FAILURE=99
+
 # hacked-up long option parsing
 for arg in $@ ; do
 	case $arg in
 		--list-subtests)
-			exit 79
+			exit $IGT_EXIT_INVALID
 			;;
 		--run-subtest)
-			exit 79
+			exit $IGT_EXIT_INVALID
+			;;
+		--debug)
+			IGT_LOG_LEVEL=debug
+			;;
+		--help-description)
+			echo $IGT_TEST_DESCRIPTION
+			exit $IGT_EXIT_SUCCESS
+			;;
+		--help)
+			echo "Usage: `basename $0` [OPTIONS]"
+			echo "  --list-subtests"
+			echo "  --run-subtest <pattern>"
+			echo "  --debug"
+			echo "  --help-description"
+			echo "  --help"
+			exit $IGT_EXIT_SUCCESS
 			;;
 	esac
 done
 
+skip() {
+	echo "$@"
+	exit $IGT_EXIT_SKIP
+}
+
 die() {
 	echo "$@"
-	exit 1
+	exit $IGT_EXIT_FAILURE
 }
 
 do_or_die() {
-	$@ > /dev/null 2>&1 || (echo "FAIL: $@ ($?)" && exit -1)
+	$@ > /dev/null 2>&1 || (echo "FAIL: $@ ($?)" && exit $IGT_EXIT_FAILURE)
 }
 
 if [ -d /debug/dri ] ; then
@@ -38,7 +65,7 @@ for minor in `seq 0 16`; do
 done
 
 if [ $i915_dfs_path = "x" ] ; then
-	die " i915 debugfs path not found."
+	skip " i915 debugfs path not found."
 fi
 
 # read everything we can
@@ -47,7 +74,7 @@ if [ `cat $i915_dfs_path/clients | wc -l` -gt "2" ] ; then
 		die "ERROR: other drm clients running"
 fi
 
-whoami | grep -q root || ( echo ERROR: not running as root; exit 1 )
+whoami | grep -q root || ( echo ERROR: not running as root; exit $IGT_EXIT_FAILURE )
 
 i915_sfs_path=
 if [ -d /sys/class/drm ] ; then
@@ -60,7 +87,7 @@ fi
 
 function drmtest_skip_on_simulation()
 {
-	[ -n "$INTEL_SIMULATION" ] && exit 77
+	[ -n "$INTEL_SIMULATION" ] && exit $IGT_EXIT_SKIP
 }
 
 drmtest_skip_on_simulation
