@@ -27,19 +27,17 @@
  *
  */
 
+#include "igt.h"
 #include <stdlib.h>
 
-#include "drmtest.h"
-#include "igt_aux.h"
 
-struct igt_eviction_test_ops
-{
+struct igt_eviction_test_ops {
 	uint32_t (*create)(int fd, uint64_t size);
 	void (*flink)(uint32_t old_handle, uint32_t new_handle);
 	void	 (*close)(int fd, uint32_t bo);
 	int	 (*copy)(int fd, uint32_t dst, uint32_t src,
 			 uint32_t *all_bo, int nr_bos);
-	void	 (*clear)(int fd, uint32_t bo, int size);
+	void	 (*clear)(int fd, uint32_t bo, uint64_t size);
 };
 
 #define FORKING_EVICTIONS_INTERRUPTIBLE	  (1 << 0)
@@ -59,11 +57,12 @@ static void exchange_uint32_t(void *array, unsigned i, unsigned j)
 }
 
 static int minor_evictions(int fd, struct igt_eviction_test_ops *ops,
-			   int surface_size, int nr_surfaces)
+			   uint64_t surface_size,
+			   uint64_t nr_surfaces)
 {
 	uint32_t *bo, *sel;
-	int n, m, pass, fail;
-	int total_surfaces;
+	uint64_t n, m, total_surfaces;
+	int pass, fail;
 
 	/* Make sure nr_surfaces is not divisible by seven
 	 * to avoid duplicates in the selection loop below.
@@ -103,11 +102,11 @@ static int minor_evictions(int fd, struct igt_eviction_test_ops *ops,
 }
 
 static int major_evictions(int fd, struct igt_eviction_test_ops *ops,
-			   int surface_size, int nr_surfaces)
+			   uint64_t surface_size, uint64_t nr_surfaces)
 {
-	int n, m, loop;
+	uint64_t n, m;
 	uint32_t *bo;
-	int ret;
+	int ret, loop;
 
 	intel_require_memory(nr_surfaces, surface_size, CHECK_RAM);
 
@@ -131,10 +130,10 @@ static int major_evictions(int fd, struct igt_eviction_test_ops *ops,
 }
 
 static void mlocked_evictions(int fd, struct igt_eviction_test_ops *ops,
-			      int surface_size,
-			      int surface_count)
+			      uint64_t surface_size,
+			      uint64_t surface_count)
 {
-	size_t sz, pin;
+	uint64_t sz, pin;
 	void *locked;
 
 	intel_require_memory(surface_count, surface_size, CHECK_RAM);
@@ -143,10 +142,11 @@ static void mlocked_evictions(int fd, struct igt_eviction_test_ops *ops,
 	pin = intel_get_avail_ram_mb();
 	pin *= 1024 * 1024;
 	igt_require(pin > sz);
-	pin -= 3*sz/2;
+	pin -= sz;
 
-	igt_debug("Pinning [%ld, %ld] MiB\n",
-		  (long)pin/(1024*1024), (long)(pin + sz)/(1024*1024));
+	igt_debug("Pinning [%'lld, %'lld] MiB\n",
+		  (long long)pin/(1024*1024),
+		  (long long)(pin + sz)/(1024*1024));
 
 	locked = malloc(pin + sz);
 	if (locked != NULL && mlock(locked, pin + sz)) {
@@ -160,7 +160,8 @@ static void mlocked_evictions(int fd, struct igt_eviction_test_ops *ops,
 
 	igt_fork(child, 1) {
 		uint32_t *bo;
-		int n, ret;
+		uint64_t n;
+		int ret;
 
 		bo = malloc(surface_count*sizeof(*bo));
 		igt_assert(bo);
@@ -185,7 +186,7 @@ static void mlocked_evictions(int fd, struct igt_eviction_test_ops *ops,
 
 			locked = malloc(surface_size);
 			if (locked == NULL || mlock(locked, surface_size))
-				exit(ENOSPC);
+				free(locked);
 		}
 
 		for (n = 0; n < surface_count; n++)
@@ -196,12 +197,13 @@ static void mlocked_evictions(int fd, struct igt_eviction_test_ops *ops,
 }
 
 static int swapping_evictions(int fd, struct igt_eviction_test_ops *ops,
-			      int surface_size,
-			      int working_surfaces,
-			      int trash_surfaces)
+			      uint64_t surface_size,
+			      uint64_t working_surfaces,
+			      uint64_t trash_surfaces)
 {
 	uint32_t *bo;
-	int i, n, pass, ret;
+	uint64_t i, n;
+	int pass, ret;
 
 	intel_require_memory(working_surfaces, surface_size, CHECK_RAM);
 
@@ -233,13 +235,15 @@ static int swapping_evictions(int fd, struct igt_eviction_test_ops *ops,
 }
 
 static int forking_evictions(int fd, struct igt_eviction_test_ops *ops,
-				int surface_size, int working_surfaces,
-				int trash_surfaces, unsigned flags)
+			     uint64_t surface_size,
+			     uint64_t working_surfaces,
+			     uint64_t trash_surfaces,
+			     unsigned flags)
 {
+	const int num_threads = sysconf(_SC_NPROCESSORS_ONLN);
+	uint64_t bo_count, n, l;
 	uint32_t *bo;
-	int n, pass, l, ret;
-	int num_threads = sysconf(_SC_NPROCESSORS_ONLN);
-	int bo_count;
+	int pass, ret;
 
 	intel_require_memory(working_surfaces, surface_size, CHECK_RAM);
 
@@ -273,7 +277,7 @@ static int forking_evictions(int fd, struct igt_eviction_test_ops *ops,
 		igt_permute_array(bo, bo_count, exchange_uint32_t);
 
 		if (flags & FORKING_EVICTIONS_DUP_DRMFD) {
-			realfd = drm_open_any();
+			realfd = drm_open_driver(DRIVER_INTEL);
 
 			/* We can overwrite the bo array since we're forked. */
 			for (l = 0; l < bo_count; l++) {
