@@ -30,6 +30,7 @@
  *  test we can't submit contexts to unsupported rings
  */
 
+#include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -42,8 +43,9 @@
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
+
+IGT_TEST_DESCRIPTION("Test that context cannot be submitted to unsupported"
+		     " rings.");
 
 /* Copied from gem_exec_nop.c */
 static int exec(int fd, uint32_t handle, int ring, int ctx_id)
@@ -90,7 +92,7 @@ igt_main
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		fd = drm_open_any_render();
+		fd = drm_open_driver_render(DRIVER_INTEL);
 
 		ctx_id = gem_context_create(fd);
 
