@@ -29,6 +29,7 @@
  * This test is useful for finding memory and refcount leaks.
  */
 
+#include "igt.h"
 #include <pthread.h>
 #include <stdlib.h>
 #include <sys/ioctl.h>
@@ -40,12 +41,7 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
 #include "intel_bufmgr.h"
-#include "intel_batchbuffer.h"
-#include "intel_io.h"
-#include "intel_chipset.h"
 
 IGT_TEST_DESCRIPTION("Basic test for memory and refcount leaks.");
 
@@ -81,7 +77,7 @@ static void *work(void *arg)
 	int i;
 
 	if (multiple_fds)
-		td_fd = fd = drm_open_any_render();
+		td_fd = fd = drm_open_driver_render(DRIVER_INTEL);
 	else
 		td_fd = fd;
 
@@ -147,7 +143,7 @@ int main(int argc, char *argv[])
 	igt_simple_init_parse_opts(&argc, argv, "i:c:n:mu", NULL, NULL,
 				   opt_handler, NULL);
 
-	fd = drm_open_any_render();
+	fd = drm_open_driver_render(DRIVER_INTEL);
 	devid = intel_get_drm_devid(fd);
 
 	if (igt_run_in_simulation()) {
