@@ -130,10 +130,9 @@ copy(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo)
 	return ret;
 }
 
-static void clear(int fd, uint32_t handle, int size)
+static void clear(int fd, uint32_t handle, uint64_t size)
 {
-	void *base = gem_mmap__cpu(fd, handle, 0, size, PROT_READ | PROT_WRITE);
-
+	void *base = gem_mmap__cpu(fd, handle, 0, size, PROT_WRITE);
 	memset(base, 0, size);
 	munmap(base, size);
 }
@@ -183,7 +182,9 @@ static void test_major_evictions(int fd, int size, int count)
 
 igt_main
 {
-	int size, count, fd;
+	uint64_t size, count;
+	int fd;
+
 	size = count = 0;
 	fd = -1;
 
@@ -243,32 +244,32 @@ igt_main
 		test_major_evictions(fd, size, count);
 	}
 
-	if (igt_fork_hang_helper()) {
-		igt_fixture {
-			size = 1024 * 1024;
-			count = 3*gem_aperture_size(fd) / size / 4;
-		}
+	igt_fixture {
+		igt_fork_hang_helper();
 
-		igt_subtest("mlocked-hang")
-			test_mlocked_evictions(fd, size, count);
-
-		igt_subtest("swapping-hang")
-			test_swapping_evictions(fd, size, count);
-
-		igt_subtest("minor-hang")
-			test_minor_evictions(fd, size, count);
-
-		igt_subtest("major-hang") {
-			size = 3*gem_aperture_size(fd) / 4;
-			count = 4;
-			test_major_evictions(fd, size, count);
-		}
-
-		igt_stop_hang_helper();
+		size = 1024 * 1024;
+		count = 3*gem_aperture_size(fd) / size / 4;
 	}
-	igt_stop_signal_helper();
+
+	igt_subtest("mlocked-hang")
+		test_mlocked_evictions(fd, size, count);
+
+	igt_subtest("swapping-hang")
+		test_swapping_evictions(fd, size, count);
+
+	igt_subtest("minor-hang")
+		test_minor_evictions(fd, size, count);
+
+	igt_subtest("major-hang") {
+		size = 3*gem_aperture_size(fd) / 4;
+		count = 4;
+		test_major_evictions(fd, size, count);
+	}
+
+	igt_stop_hang_helper();
 
 	igt_fixture {
+		igt_stop_signal_helper();
 		close(fd);
 	}
 }
