@@ -35,14 +35,6 @@
 #define I915_PARAM_CMD_PARSER_VERSION       28
 #endif
 
-static int __gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
-{
-	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, execbuf))
-		return -errno;
-
-	return 0;
-}
-
 static void exec_batch_patched(int fd, uint32_t cmd_bo, uint32_t *cmds,
 			       int size, int patch_offset, uint64_t expected_value)
 {
@@ -306,7 +298,7 @@ igt_main
 		rc = drmIoctl(fd, DRM_IOCTL_I915_GETPARAM, &gp);
 		igt_require(!rc && parser_version > 0);
 
-		igt_require(gem_uses_aliasing_ppgtt(fd));
+		igt_require(gem_uses_ppgtt(fd));
 
 		handle = gem_create(fd, 4096);
 
