@@ -25,6 +25,7 @@
  */
 
 #define _GNU_SOURCE
+#include "igt.h"
 #include <sys/ioctl.h>
 #include <stdlib.h>
 #include <string.h>
@@ -33,10 +34,9 @@
 #include <pthread.h>
 #include <errno.h>
 
-#include "drmtest.h"
-#include "ioctl_wrappers.h"
 #include "intel_bufmgr.h"
-#include "igt_debugfs.h"
+
+IGT_TEST_DESCRIPTION("Check for flink/open vs. gem close races.");
 
 /* Testcase: check for flink/open vs. gem close races
  *
@@ -44,32 +44,11 @@
  * in the flink name and corresponding reference getting leaked.
  */
 
-/* We want lockless and I'm to lazy to dig out an atomic libarary. On x86 this
+/* We want lockless and I'm to lazy to dig out an atomic library. On x86 this
  * works, too. */
 volatile int pls_die = 0;
 int fd;
 
-static int get_object_count(void)
-{
-	FILE *file;
-	int ret, scanned;
-	int device = drm_get_card();
-	char *path;
-
-	igt_drop_caches_set(DROP_RETIRE);
-
-	ret = asprintf(&path, "/sys/kernel/debug/dri/%d/i915_gem_objects", device);
-	igt_assert(ret != -1);
-
-	file = fopen(path, "r");
-
-	scanned = fscanf(file, "%i objects", &ret);
-	igt_assert(scanned == 1);
-
-	return ret;
-}
-
-
 static void *thread_fn_flink_name(void *p)
 {
 	struct drm_gem_open open_struct;
@@ -105,13 +84,12 @@ static void test_flink_name(void)
 
 	threads = calloc(num_threads, sizeof(pthread_t));
 
-	fd = drm_open_any();
-	igt_assert(fd >= 0);
+	fd = drm_open_driver(DRIVER_INTEL);
 
 	for (i = 0; i < num_threads; i++) {
 		r = pthread_create(&threads[i], NULL,
 				   thread_fn_flink_name, NULL);
-		igt_assert(r == 0);
+		igt_assert_eq(r, 0);
 	}
 
 	for (i = 0; i < 1000000; i++) {
@@ -167,21 +145,20 @@ static void test_flink_close(void)
 
 	/* Allocate exit handler fds in here so that we dont screw
 	 * up the counts */
-	fake = drm_open_any();
+	fake = drm_open_driver(DRIVER_INTEL);
 
-	obj_count = get_object_count();
+	obj_count = igt_get_stable_obj_count(fake);
 
 	num_threads = sysconf(_SC_NPROCESSORS_ONLN);
 
 	threads = calloc(num_threads, sizeof(pthread_t));
 
-	fd = drm_open_any();
-	igt_assert(fd >= 0);
+	fd = drm_open_driver(DRIVER_INTEL);
 
 	for (i = 0; i < num_threads; i++) {
 		r = pthread_create(&threads[i], NULL,
 				   thread_fn_flink_close, NULL);
-		igt_assert(r == 0);
+		igt_assert_eq(r, 0);
 	}
 
 	sleep(5);
@@ -195,13 +172,13 @@ static void test_flink_close(void)
 
 	close(fd);
 
-	obj_count = get_object_count() - obj_count;
+	obj_count = igt_get_stable_obj_count(fake) - obj_count;
 
 	igt_info("leaked %i objects\n", obj_count);
 
 	close(fake);
 
-	igt_assert(obj_count == 0);
+	igt_assert_eq(obj_count, 0);
 }
 
 igt_main
