@@ -140,10 +140,12 @@ static void run(data_t *data, int child)
 	 * set-to-gtt-domain within the fault handler.
 	 */
 	if (write) {
-		ptr = gem_mmap(data->fd, handle, size, PROT_READ | PROT_WRITE);
+		ptr = gem_mmap__gtt(data->fd, handle, size,
+				    PROT_READ | PROT_WRITE);
 		ptr[rand() % (size / 4)] = canary;
-	} else
-		ptr = gem_mmap(data->fd, handle, size, PROT_READ);
+	} else {
+		ptr = gem_mmap__gtt(data->fd, handle, size, PROT_READ);
+	}
 	x = ptr[rand() % (size / 4)];
 	munmap(ptr, size);
 
@@ -172,6 +174,6 @@ igt_simple_main
 	igt_waitchildren();
 	gettimeofday(&end, NULL);
 
-	igt_info("Time to execute %lu children:		%7.3fms\n",
+	igt_info("Time to execute %zu children:		%7.3fms\n",
 		 ARRAY_SIZE(children), elapsed(&start, &end) / 1000);
 }
