@@ -26,6 +26,7 @@
  *
  */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -35,12 +36,7 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
 #include "intel_bufmgr.h"
-#include "intel_chipset.h"
-#include "intel_batchbuffer.h"
-#include "intel_io.h"
 
 static drm_intel_bufmgr *bufmgr;
 struct intel_batchbuffer *batch;
@@ -80,7 +76,7 @@ int main(int argc, char **argv)
 
 	bad_pipe = atoi(argv[1]);
 
-	fd = drm_open_any();
+	fd = drm_open_driver(DRIVER_INTEL);
 
 	bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 	drm_intel_bufmgr_gem_enable_reuse(bufmgr);
@@ -93,5 +89,5 @@ int main(int argc, char **argv)
 
 	close(fd);
 
-	return 0;
+	igt_exit();
 }
