@@ -26,6 +26,7 @@
  *
  */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -35,13 +36,10 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
 #include "intel_bufmgr.h"
-#include "intel_batchbuffer.h"
-#include "intel_io.h"
 #include "i830_reg.h"
-#include "intel_chipset.h"
+
+IGT_TEST_DESCRIPTION("Basic check of ring<->ring sync using a dummy reloc.");
 
 static drm_intel_bufmgr *bufmgr;
 struct intel_batchbuffer *batch;
@@ -108,7 +106,7 @@ igt_simple_main
 	int devid;
 	int i;
 
-	fd = drm_open_any();
+	fd = drm_open_driver(DRIVER_INTEL);
 	devid = intel_get_drm_devid(fd);
 	gem_require_ring(fd, I915_EXEC_BLT);
 
@@ -129,7 +127,7 @@ igt_simple_main
 
 		for (i = 0; i < NUM_FD; i++) {
 			sprintf(buffer_name, "Target buffer %d\n", i);
-			mfd[i] = drm_open_any();
+			mfd[i] = drm_open_driver(DRIVER_INTEL);
 			mbufmgr[i] = drm_intel_bufmgr_gem_init(mfd[i], 4096);
 			igt_assert_f(mbufmgr[i],
 				     "fail to initialize buf manager for drm_fd %d\n",
