@@ -34,6 +34,7 @@
  * ranged-buffer-flush paths in the kernel.
  */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -45,12 +46,6 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_chipset.h"
-#include "intel_io.h"
-#include "igt_aux.h"
-#include "igt_gt.h"
 
 IGT_TEST_DESCRIPTION("Test pread behavior when getting values out of"
 		     " just-drawn-to buffers.");
@@ -216,7 +211,7 @@ igt_main
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
 
 		bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 		drm_intel_bufmgr_gem_enable_reuse(bufmgr);
@@ -238,10 +233,8 @@ igt_main
 			do_test(fd, t->cache, src, start, dst, 100, no_hang);
 		igt_stop_signal_helper();
 
-		igt_subtest_f("%s-hang", t->name) {
-			igt_require_hang_ring(fd, -1);
+		igt_subtest_f("%s-hang", t->name)
 			do_test(fd, t->cache, src, start, dst, 1, bcs_hang);
-		}
 	}
 
 	igt_fixture {
