@@ -26,6 +26,7 @@
  */
 
 #define _GNU_SOURCE
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -39,13 +40,6 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_chipset.h"
-#include "intel_io.h"
-#include "igt_debugfs.h"
-#include "igt_aux.h"
-#include "igt_gt.h"
 
 IGT_TEST_DESCRIPTION("Test kernel relocations vs. gpu races.");
 
@@ -168,9 +162,8 @@ static void reloc_and_emit(int fd, drm_intel_bo *target_bo, bool faulting_reloc)
 
 	handle_relocs = gem_create(fd, 4096);
 	gem_write(fd, handle_relocs, 0, reloc, sizeof(reloc));
-	gtt_relocs = gem_mmap(fd, handle_relocs, 4096,
-			      PROT_READ | PROT_WRITE);
-	igt_assert(gtt_relocs);
+	gtt_relocs = gem_mmap__gtt(fd, handle_relocs, 4096,
+				   PROT_READ | PROT_WRITE);
 
 	exec[1].handle = special_bo->handle;
 	exec[1].relocation_count = 1;
@@ -279,7 +272,7 @@ static void do_forked_test(int fd, unsigned flags)
 
 	igt_fork(i, num_threads * 4) {
 		/* re-create process local data */
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
 		bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 		batch = intel_batchbuffer_alloc(bufmgr, devid);
 
@@ -307,7 +300,7 @@ igt_main
 	memset(blob, 'A', sizeof(blob));
 
 	igt_fixture {
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
 		bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 		/* disable reuse, otherwise the test fails */
 		//drm_intel_bufmgr_gem_enable_reuse(bufmgr);
@@ -325,18 +318,14 @@ igt_main
 	igt_subtest("interruptible")
 		do_test(fd, false, no_hang);
 
-	igt_subtest("interruptible-hang") {
-		igt_require_hang_ring(fd, I915_EXEC_BLT);
+	igt_subtest("interruptible-hang")
 		do_test(fd, false, bcs_hang);
-	}
 
 	igt_subtest("faulting-reloc-interruptible")
 		do_test(fd, true, no_hang);
 
-	igt_subtest("faulting-reloc-interruptible-hang") {
-		igt_require_hang_ring(fd, I915_EXEC_BLT);
+	igt_subtest("faulting-reloc-interruptible-hang")
 		do_test(fd, true, bcs_hang);
-	}
 	igt_stop_signal_helper();
 
 	for (unsigned flags = 0; flags <= ALL_FLAGS; flags++) {
