@@ -33,6 +33,7 @@
  * The goal is to simply ensure the basics work.
  */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <sys/ioctl.h>
 #include <stdio.h>
@@ -45,13 +46,7 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
 #include "intel_bufmgr.h"
-#include "intel_batchbuffer.h"
-#include "intel_io.h"
-#include "intel_chipset.h"
-#include "igt_aux.h"
 
 #define WIDTH 512
 #define STRIDE (WIDTH*4)
@@ -205,7 +200,7 @@ igt_main
 	int count = 0;
 
 	igt_fixture {
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
 	}
 
 	igt_subtest("basic") {
@@ -228,6 +223,4 @@ igt_main
 		intel_require_memory(count, SIZE, CHECK_RAM | CHECK_SWAP);
 		run_test(fd, count);
 	}
-
-	igt_exit();
 }
