@@ -36,7 +36,7 @@
  */
 
 /*
- * Testcase: Test swizzling by testing pwrite does the invers of pread
+ * Testcase: Test swizzling by testing pwrite does the inverse of pread
  *
  * Together with the explicit pread testcase, this should cover our swizzle
  * handling.
@@ -44,6 +44,7 @@
  * Note that this test will use swap in an effort to test all of ram.
  */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -53,10 +54,12 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/ioctl.h>
-#include "drm.h"
-#include "i915_drm.h"
-#include "drmtest.h"
-#include "intel_gpu_tools.h"
+
+#include <drm.h>
+
+
+IGT_TEST_DESCRIPTION("Test swizzling by testing pwrite does the inverse of"
+		     " pread.");
 
 #define WIDTH 512
 #define HEIGHT 512
@@ -65,22 +68,6 @@ static uint32_t current_tiling_mode;
 
 #define PAGE_SIZE 4096
 
-static void
-gem_get_tiling(int fd, uint32_t handle, uint32_t *tiling, uint32_t *swizzle)
-{
-	struct drm_i915_gem_get_tiling get_tiling;
-	int ret;
-
-	memset(&get_tiling, 0, sizeof(get_tiling));
-	get_tiling.handle = handle;
-
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_GET_TILING, &get_tiling);
-	igt_assert(ret == 0);
-
-	*tiling = get_tiling.tiling_mode;
-	*swizzle = get_tiling.swizzle_mode;
-}
-
 static uint32_t
 create_bo_and_fill(int fd)
 {
@@ -92,7 +79,8 @@ create_bo_and_fill(int fd)
 	gem_set_tiling(fd, handle, current_tiling_mode, WIDTH * sizeof(uint32_t));
 
 	/* Fill the BO with dwords starting at start_val */
-	data = gem_mmap(fd, handle, sizeof(linear), PROT_READ | PROT_WRITE);
+	data = gem_mmap__gtt(fd, handle, sizeof(linear),
+			     PROT_READ | PROT_WRITE);
 	for (i = 0; i < WIDTH*HEIGHT; i++)
 		data[i] = i;
 	munmap(data, sizeof(linear));
@@ -111,8 +99,7 @@ create_bo(int fd)
 	return handle;
 }
 
-int
-main(int argc, char **argv)
+igt_simple_main
 {
 	int fd;
 	uint32_t *data;
@@ -121,7 +108,7 @@ main(int argc, char **argv)
 	uint32_t handle, handle_target;
 	int count;
 	
-	fd = drm_open_any();
+	fd = drm_open_driver(DRIVER_INTEL);
 	count = SLOW_QUICK(intel_get_total_ram_mb() * 9 / 10, 8) ;
 
 	for (i = 0; i < count/2; i++) {
@@ -136,7 +123,7 @@ main(int argc, char **argv)
 		gem_write(fd, handle_target, 0, linear, sizeof(linear));
 
 		/* Check the target bo's contents. */
-		data = gem_mmap(fd, handle_target, sizeof(linear), PROT_READ | PROT_WRITE);
+		data = gem_mmap__gtt(fd, handle_target, sizeof(linear), PROT_READ | PROT_WRITE);
 		for (j = 0; j < WIDTH*HEIGHT; j++)
 			igt_assert_f(data[j] == j,
 				     "mismatch at %i: %i\n",
@@ -151,6 +138,4 @@ main(int argc, char **argv)
 	}
 
 	close(fd);
-
-	return 0;
 }
