@@ -44,6 +44,7 @@
  * This test obviously needs swap present (and exits if none is detected).
  */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -57,11 +58,6 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_io.h"
-#include "igt_aux.h"
-#include "igt_debugfs.h"
 
 IGT_TEST_DESCRIPTION("Exercise swizzle code for swapping.");
 
@@ -82,7 +78,7 @@ create_bo(int fd)
 	handle = gem_create(fd, LINEAR_DWORDS);
 	gem_set_tiling(fd, handle, current_tiling_mode, WIDTH * sizeof(uint32_t));
 
-	data = gem_mmap(fd, handle, LINEAR_DWORDS, PROT_READ | PROT_WRITE);
+	data = __gem_mmap__gtt(fd, handle, LINEAR_DWORDS, PROT_READ | PROT_WRITE);
 	if (data == NULL) {
 		gem_close(fd, handle);
 		return 0;
@@ -98,8 +94,8 @@ fill_bo(int fd, uint32_t handle)
 	uint32_t *data;
 	int i;
 
-	data = gem_mmap(fd, handle, LINEAR_DWORDS, PROT_READ | PROT_WRITE);
-	igt_assert(data);
+	data = gem_mmap__gtt(fd, handle, LINEAR_DWORDS,
+			     PROT_READ | PROT_WRITE);
 
 	gem_set_domain(fd, handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
 	for (i = 0; i < WIDTH*HEIGHT; i++)
@@ -113,7 +109,7 @@ check_bo(int fd, uint32_t handle)
 	uint32_t *data;
 	int j;
 
-	data = gem_mmap(fd, handle, LINEAR_DWORDS, PROT_READ);
+	data = gem_mmap__gtt(fd, handle, LINEAR_DWORDS, PROT_READ);
 	gem_set_domain(fd, handle, I915_GEM_DOMAIN_GTT, 0);
 	j = rand() % (WIDTH * HEIGHT);
 	igt_assert_f(data[j] == j, "mismatch at %i: %i\n", j, data[j]);
@@ -161,19 +157,8 @@ static void thread_fini(struct thread *t)
 
 static void check_memory_layout(void)
 {
-	FILE *tiling_debugfs_file;
-	char *line = NULL;
-	size_t sz = 0;
-
-	tiling_debugfs_file = igt_debugfs_fopen("i915_swizzle_info", "r");
-	igt_assert(tiling_debugfs_file);
-
-	while (getline(&line, &sz, tiling_debugfs_file) > 0) {
-		if (strstr(line, "L-shaped") == NULL)
-			continue;
-
-		igt_skip("L-shaped memory configuration detected\n");
-	}
+	igt_skip_on_f(igt_debugfs_search("i915_swizzle_info", "L-shaped"),
+		      "L-shaped memory configuration detected\n");
 
 	igt_debug("normal memory configuration detected, continuing\n");
 }
@@ -190,7 +175,7 @@ igt_main
 
 		intel_purge_vm_caches();
 
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
 
 		check_memory_layout();
 
