@@ -36,6 +36,7 @@
  * The goal is to simply ensure the basics work.
  */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -47,15 +48,12 @@
 #include <sys/mman.h>
 #include <signal.h>
 #include <pthread.h>
+#include <time.h>
 
 #include "drm.h"
 #include "i915_drm.h"
 
-#include "drmtest.h"
 #include "intel_bufmgr.h"
-#include "intel_batchbuffer.h"
-#include "intel_chipset.h"
-#include "ioctl_wrappers.h"
 
 #include "eviction_common.c"
 
@@ -63,17 +61,6 @@
 #define PAGE_SIZE 4096
 #endif
 
-#define LOCAL_I915_GEM_USERPTR       0x33
-#define LOCAL_IOCTL_I915_GEM_USERPTR DRM_IOWR (DRM_COMMAND_BASE + LOCAL_I915_GEM_USERPTR, struct local_i915_gem_userptr)
-struct local_i915_gem_userptr {
-	uint64_t user_ptr;
-	uint64_t user_size;
-	uint32_t flags;
-#define LOCAL_I915_USERPTR_READ_ONLY (1<<0)
-#define LOCAL_I915_USERPTR_UNSYNCHRONIZED (1<<31)
-	uint32_t handle;
-};
-
 static uint32_t userptr_flags = LOCAL_I915_USERPTR_UNSYNCHRONIZED;
 
 #define WIDTH 512
@@ -91,32 +78,6 @@ static void gem_userptr_test_synchronized(void)
 	userptr_flags = 0;
 }
 
-static int gem_userptr(int fd, void *ptr, int size, int read_only, uint32_t *handle)
-{
-	struct local_i915_gem_userptr userptr;
-	int ret;
-
-	memset(&userptr, 0, sizeof(userptr));
-	userptr.user_ptr = (uintptr_t)ptr;
-	userptr.user_size = size;
-	userptr.flags = userptr_flags;
-	if (read_only)
-		userptr.flags |= LOCAL_I915_USERPTR_READ_ONLY;
-
-	ret = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr);
-	if (ret)
-		ret = errno;
-	igt_skip_on_f(ret == ENODEV &&
-		      (userptr_flags & LOCAL_I915_USERPTR_UNSYNCHRONIZED) == 0 &&
-		      !read_only,
-		      "Skipping, synchronized mappings with no kernel CONFIG_MMU_NOTIFIER?");
-	if (ret == 0)
-		*handle = userptr.handle;
-
-	return ret;
-}
-
-
 static void gem_userptr_sync(int fd, uint32_t handle)
 {
 	gem_set_domain(fd, handle, I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
@@ -175,8 +136,8 @@ copy(int fd, uint32_t dst, uint32_t src, unsigned int error)
 	reloc[1].read_domains = I915_GEM_DOMAIN_RENDER;
 	reloc[1].write_domain = 0;
 
+	memset(&exec, 0, sizeof(exec));
 	memset(obj, 0, sizeof(obj));
-	exec.buffer_count = 0;
 	obj[exec.buffer_count++].handle = dst;
 	if (src != dst)
 		obj[exec.buffer_count++].handle = src;
@@ -185,26 +146,15 @@ copy(int fd, uint32_t dst, uint32_t src, unsigned int error)
 	obj[exec.buffer_count].relocs_ptr = (uintptr_t)reloc;
 	exec.buffer_count++;
 	exec.buffers_ptr = (uintptr_t)obj;
-
-	exec.batch_start_offset = 0;
-	exec.batch_len = i * 4;
-	exec.DR1 = exec.DR4 = 0;
-	exec.num_cliprects = 0;
-	exec.cliprects_ptr = 0;
 	exec.flags = HAS_BLT_RING(intel_get_drm_devid(fd)) ? I915_EXEC_BLT : 0;
-	i915_execbuffer2_set_context_id(exec, 0);
-	exec.rsvd2 = 0;
 
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &exec);
-	if (ret)
-		ret = errno;
+	ret = __gem_execbuf(fd, &exec);
+	gem_close(fd, handle);
 
 	if (error == ~0)
-		igt_assert(ret != 0);
+		igt_assert_neq(ret, 0);
 	else
-		igt_assert(ret == error);
-
-	gem_close(fd, handle);
+		igt_assert_eq(ret, -error);
 }
 
 static int
@@ -259,6 +209,7 @@ blit(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo)
 	reloc[1].read_domains = I915_GEM_DOMAIN_RENDER;
 	reloc[1].write_domain = 0;
 
+	memset(&exec, 0, sizeof(exec));
 	obj = calloc(n_bo + 1, sizeof(*obj));
 	for (n = 0; n < n_bo; n++)
 		obj[n].handle = all_bo[n];
@@ -268,19 +219,9 @@ blit(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo)
 
 	exec.buffers_ptr = (uintptr_t)obj;
 	exec.buffer_count = n_bo + 1;
-	exec.batch_start_offset = 0;
-	exec.batch_len = i * 4;
-	exec.DR1 = exec.DR4 = 0;
-	exec.num_cliprects = 0;
-	exec.cliprects_ptr = 0;
 	exec.flags = HAS_BLT_RING(intel_get_drm_devid(fd)) ? I915_EXEC_BLT : 0;
-	i915_execbuffer2_set_context_id(exec, 0);
-	exec.rsvd2 = 0;
-
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &exec);
-	if (ret)
-		ret = errno;
 
+	ret = __gem_execbuf(fd, &exec);
 	gem_close(fd, handle);
 	free(obj);
 
@@ -291,10 +232,9 @@ static uint32_t
 create_userptr(int fd, uint32_t val, uint32_t *ptr)
 {
 	uint32_t handle;
-	int i, ret;
+	int i;
 
-	ret = gem_userptr(fd, ptr, sizeof(linear), 0, &handle);
-	igt_assert(ret == 0);
+	gem_userptr(fd, ptr, sizeof(linear), 0, userptr_flags, &handle);
 	igt_assert(handle != 0);
 
 	/* Fill the BO with dwords starting at val */
@@ -361,11 +301,10 @@ static void free_handle_ptr(uint32_t handle)
 	handle_ptr_map[handle] = NULL;
 }
 
-static uint32_t create_userptr_bo(int fd, int size)
+static uint32_t create_userptr_bo(int fd, uint64_t size)
 {
 	void *ptr;
 	uint32_t handle;
-	int ret;
 
 	ptr = mmap(NULL, size,
 		   PROT_READ | PROT_WRITE,
@@ -373,8 +312,7 @@ static uint32_t create_userptr_bo(int fd, int size)
 		   -1, 0);
 	igt_assert(ptr != MAP_FAILED);
 
-	ret = gem_userptr(fd, (uint32_t *)ptr, size, 0, &handle);
-	igt_assert(ret == 0);
+	gem_userptr(fd, (uint32_t *)ptr, size, 0, userptr_flags, &handle);
 	add_handle_ptr(handle, ptr, size);
 
 	return handle;
@@ -390,7 +328,7 @@ static void flink_userptr_bo(uint32_t old_handle, uint32_t new_handle)
 		       handle_size_map[old_handle]);
 }
 
-static void clear(int fd, uint32_t handle, int size)
+static void clear(int fd, uint32_t handle, uint64_t size)
 {
 	void *ptr = get_handle_ptr(handle);
 
@@ -452,7 +390,7 @@ static int has_userptr(int fd)
 	igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE) == 0);
 	oldflags = userptr_flags;
 	gem_userptr_test_unsynchronized();
-	ret = gem_userptr(fd, ptr, PAGE_SIZE, 0, &handle);
+	ret = __gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle);
 	userptr_flags = oldflags;
 	if (ret != 0) {
 		free(ptr);
@@ -476,7 +414,7 @@ static int test_input_checking(int fd)
 	userptr.user_size = 0;
 	userptr.flags = ~0;
 	ret = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr);
-	igt_assert(ret != 0);
+	igt_assert_neq(ret, 0);
 
 	/* Too big. */
 	memset(&userptr, 0, sizeof(userptr));
@@ -484,7 +422,7 @@ static int test_input_checking(int fd)
 	userptr.user_size = ~0;
 	userptr.flags = 0;
 	ret = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr);
-	igt_assert(ret != 0);
+	igt_assert_neq(ret, 0);
 
 	/* Both wrong. */
 	memset(&userptr, 0, sizeof(userptr));
@@ -492,7 +430,7 @@ static int test_input_checking(int fd)
 	userptr.user_size = ~0;
 	userptr.flags = ~0;
 	ret = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr);
-	igt_assert(ret != 0);
+	igt_assert_neq(ret, 0);
 
 	return 0;
 }
@@ -511,7 +449,7 @@ static int test_access_control(int fd)
 
 		igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE) == 0);
 
-		ret = gem_userptr(fd, ptr, PAGE_SIZE, 0, &handle);
+		ret = __gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle);
 		if (ret == 0)
 			gem_close(fd, handle);
 		free(ptr);
@@ -526,11 +464,9 @@ static int test_access_control(int fd)
 static int test_invalid_null_pointer(int fd)
 {
 	uint32_t handle;
-	int ret;
 
 	/* NULL pointer. */
-	ret = gem_userptr(fd, NULL, PAGE_SIZE, 0, &handle);
-	igt_assert(ret == 0);
+	gem_userptr(fd, NULL, PAGE_SIZE, 0, userptr_flags, &handle);
 
 	copy(fd, handle, handle, ~0); /* QQQ Precise errno? */
 	gem_close(fd, handle);
@@ -542,18 +478,16 @@ static int test_invalid_gtt_mapping(int fd)
 {
 	uint32_t handle, handle2;
 	void *ptr;
-	int ret;
 
 	/* GTT mapping */
 	handle = create_bo(fd, 0);
-	ptr = gem_mmap__gtt(fd, handle, sizeof(linear), PROT_READ | PROT_WRITE);
+	ptr = gem_mmap__gtt(fd, handle, sizeof(linear),
+			    PROT_READ | PROT_WRITE);
 	gem_close(fd, handle);
-	igt_assert(ptr != NULL);
 	igt_assert(((unsigned long)ptr & (PAGE_SIZE - 1)) == 0);
 	igt_assert((sizeof(linear) & (PAGE_SIZE - 1)) == 0);
 
-	ret = gem_userptr(fd, ptr, sizeof(linear), 0, &handle2);
-	igt_assert(ret == 0);
+	gem_userptr(fd, ptr, sizeof(linear), 0, userptr_flags, &handle2);
 	copy(fd, handle2, handle2, ~0); /* QQQ Precise errno? */
 	gem_close(fd, handle2);
 
@@ -566,13 +500,16 @@ static int test_invalid_gtt_mapping(int fd)
 #define PE_BUSY 0x2
 static void test_process_exit(int fd, int flags)
 {
+	if (flags & PE_GTT_MAP)
+		igt_require(gem_has_llc(fd));
+
 	igt_fork(child, 1) {
 		uint32_t handle;
 
 		handle = create_userptr_bo(fd, sizeof(linear));
 
 		if (flags & PE_GTT_MAP) {
-			uint32_t *ptr = gem_mmap__gtt(fd, handle, sizeof(linear), PROT_READ | PROT_WRITE);
+			uint32_t *ptr = __gem_mmap__gtt(fd, handle, sizeof(linear), PROT_READ | PROT_WRITE);
 			if (ptr)
 				*ptr = 0;
 		}
@@ -590,16 +527,18 @@ static void test_forked_access(int fd)
 	int ret;
 
 	ret = posix_memalign(&ptr1, PAGE_SIZE, sizeof(linear));
+#ifdef MADV_DONTFORK
 	ret |= madvise(ptr1, sizeof(linear), MADV_DONTFORK);
-	ret |= gem_userptr(fd, ptr1, sizeof(linear), 0, &handle1);
-	igt_assert(ret == 0);
+#endif
+	gem_userptr(fd, ptr1, sizeof(linear), 0, userptr_flags, &handle1);
 	igt_assert(ptr1);
 	igt_assert(handle1);
 
 	ret = posix_memalign(&ptr2, PAGE_SIZE, sizeof(linear));
+#ifdef MADV_DONTFORK
 	ret |= madvise(ptr2, sizeof(linear), MADV_DONTFORK);
-	ret |= gem_userptr(fd, ptr2, sizeof(linear), 0, &handle2);
-	igt_assert(ret == 0);
+#endif
+	gem_userptr(fd, ptr2, sizeof(linear), 0, userptr_flags, &handle2);
 	igt_assert(ptr2);
 	igt_assert(handle2);
 
@@ -619,12 +558,16 @@ static void test_forked_access(int fd)
 
 	igt_assert(memcmp(ptr1, ptr2, sizeof(linear)) == 0);
 
+#ifdef MADV_DOFORK
 	ret = madvise(ptr1, sizeof(linear), MADV_DOFORK);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
+#endif
 	free(ptr1);
 
+#ifdef MADV_DOFORK
 	ret = madvise(ptr2, sizeof(linear), MADV_DOFORK);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
+#endif
 	free(ptr2);
 }
 
@@ -642,8 +585,7 @@ static int test_forbidden_ops(int fd)
 
 	igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE) == 0);
 
-	ret = gem_userptr(fd, ptr, PAGE_SIZE, 0, &handle);
-	igt_assert(ret == 0);
+	gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle);
 
 	/* pread/pwrite are not always forbidden, but when they
 	 * are they should fail with EINVAL.
@@ -668,24 +610,79 @@ static int test_forbidden_ops(int fd)
 	return 0;
 }
 
+static void test_relocations(int fd)
+{
+	struct drm_i915_gem_relocation_entry *reloc;
+	struct drm_i915_gem_exec_object2 obj;
+	struct drm_i915_gem_execbuffer2 exec;
+	unsigned size;
+	void *ptr;
+	int i;
+
+	size = PAGE_SIZE + ALIGN(sizeof(*reloc)*256, PAGE_SIZE);
+
+	memset(&obj, 0, sizeof(obj));
+	igt_assert(posix_memalign(&ptr, PAGE_SIZE, size) == 0);
+	gem_userptr(fd, ptr, size, 0, userptr_flags, &obj.handle);
+	if (!gem_has_llc(fd))
+		gem_set_caching(fd, obj.handle, 0);
+	*(uint32_t *)ptr = MI_BATCH_BUFFER_END;
+
+	reloc = (typeof(reloc))((char *)ptr + PAGE_SIZE);
+	obj.relocs_ptr = (uintptr_t)reloc;
+	obj.relocation_count = 256;
+
+	memset(reloc, 0, 256*sizeof(*reloc));
+	for (i = 0; i < 256; i++) {
+		reloc[i].offset = 2048 - 4*i;
+		reloc[i].target_handle = obj.handle;
+		reloc[i].read_domains = I915_GEM_DOMAIN_INSTRUCTION;
+	}
+
+	memset(&exec, 0, sizeof(exec));
+	exec.buffers_ptr = (uintptr_t)&obj;
+	exec.buffer_count = 1;
+	gem_execbuf(fd, &exec);
+
+	gem_sync(fd, obj.handle);
+	gem_close(fd, obj.handle);
+	free(ptr);
+}
+
 static unsigned char counter;
 
 static void (* volatile orig_sigbus)(int sig, siginfo_t *info, void *param);
 static volatile unsigned long sigbus_start;
 static volatile long sigbus_cnt = -1;
 
+static void *umap(int fd, uint32_t handle)
+{
+	void *ptr;
+
+	if (gem_has_llc(fd)) {
+		ptr = gem_mmap__gtt(fd, handle, sizeof(linear),
+				    PROT_READ | PROT_WRITE);
+	} else {
+		uint32_t tmp = gem_create(fd, sizeof(linear));
+		copy(fd, tmp, handle, 0);
+		ptr = gem_mmap__cpu(fd, tmp, 0, sizeof(linear), PROT_READ);
+		gem_close(fd, tmp);
+	}
+
+	return ptr;
+}
+
 static void
 check_bo(int fd1, uint32_t handle1, int is_userptr, int fd2, uint32_t handle2)
 {
 	unsigned char *ptr1, *ptr2;
 	unsigned long size = sizeof(linear);
 
-	ptr2 = gem_mmap(fd2, handle2, sizeof(linear), PROT_READ | PROT_WRITE);
-
+	ptr2 = umap(fd2, handle2);
 	if (is_userptr)
 		ptr1 = is_userptr > 0 ? get_handle_ptr(handle1) : ptr2;
 	else
-		ptr1 = gem_mmap(fd1, handle1, sizeof(linear), PROT_READ | PROT_WRITE);
+		ptr1 = umap(fd1, handle1);
 
 	igt_assert(ptr1);
 	igt_assert(ptr2);
@@ -693,10 +690,11 @@ check_bo(int fd1, uint32_t handle1, int is_userptr, int fd2, uint32_t handle2)
 	sigbus_start = (unsigned long)ptr2;
 	igt_assert(memcmp(ptr1, ptr2, sizeof(linear)) == 0);
 
-	counter++;
-
-	memset(ptr1, counter, size);
-	memset(ptr2, counter, size);
+	if (gem_has_llc(fd1)) {
+		counter++;
+		memset(ptr1, counter, size);
+		memset(ptr2, counter, size);
+	}
 
 	if (!is_userptr)
 		munmap(ptr1, sizeof(linear));
@@ -750,9 +748,8 @@ static int test_dmabuf(void)
 	uint32_t handle, handle_import;
 	int dma_buf_fd = -1;
 	int ret;
-	struct sigaction sigact, orig_sigact;
 
-	fd1 = drm_open_any();
+	fd1 = drm_open_driver(DRIVER_INTEL);
 
 	handle = create_userptr_bo(fd1, sizeof(linear));
 	memset(get_handle_ptr(handle), counter, sizeof(linear));
@@ -764,11 +761,11 @@ static int test_dmabuf(void)
 		close(fd1);
 		return 0;
 	} else {
-		igt_assert(ret == 0);
-		igt_assert(dma_buf_fd >= 0);
+		igt_assert_eq(ret, 0);
+		igt_assert_lte(0, dma_buf_fd);
 	}
 
-	fd2 = drm_open_any();
+	fd2 = drm_open_driver(DRIVER_INTEL);
 	handle_import = prime_fd_to_handle(fd2, dma_buf_fd);
 	check_bo(fd1, handle, 1, fd2, handle_import);
 
@@ -780,20 +777,24 @@ static int test_dmabuf(void)
 	free_userptr_bo(fd1, handle);
 	close(fd1);
 
-	memset(&sigact, 0, sizeof(sigact));
-	sigact.sa_sigaction = sigbus;
-	sigact.sa_flags = SA_SIGINFO;
-	ret = sigaction(SIGBUS, &sigact, &orig_sigact);
-	igt_assert(ret == 0);
+	if (gem_has_llc(fd2)) {
+		struct sigaction sigact, orig_sigact;
 
-	orig_sigbus = orig_sigact.sa_sigaction;
+		memset(&sigact, 0, sizeof(sigact));
+		sigact.sa_sigaction = sigbus;
+		sigact.sa_flags = SA_SIGINFO;
+		ret = sigaction(SIGBUS, &sigact, &orig_sigact);
+		igt_assert_eq(ret, 0);
 
-	sigbus_cnt = 0;
-	check_bo(fd2, handle_import, -1, fd2, handle_import);
-	igt_assert(sigbus_cnt > 0);
+		orig_sigbus = orig_sigact.sa_sigaction;
 
-	ret = sigaction(SIGBUS, &orig_sigact, NULL);
-	igt_assert(ret == 0);
+		sigbus_cnt = 0;
+		check_bo(fd2, handle_import, -1, fd2, handle_import);
+		igt_assert(sigbus_cnt > 0);
+
+		ret = sigaction(SIGBUS, &orig_sigact, NULL);
+		igt_assert_eq(ret, 0);
+	}
 
 	close(fd2);
 	reset_handle_ptr();
@@ -810,39 +811,52 @@ static int test_usage_restrictions(int fd)
 	igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE * 2) == 0);
 
 	/* Address not aligned. */
-	ret = gem_userptr(fd, (char *)ptr + 1, PAGE_SIZE, 0, &handle);
-	igt_assert(ret != 0);
+	ret = __gem_userptr(fd, (char *)ptr + 1, PAGE_SIZE, 0, userptr_flags, &handle);
+	igt_assert_neq(ret, 0);
 
 	/* Size not rounded to page size. */
-	ret = gem_userptr(fd, ptr, PAGE_SIZE - 1, 0, &handle);
-	igt_assert(ret != 0);
+	ret = __gem_userptr(fd, ptr, PAGE_SIZE - 1, 0, userptr_flags, &handle);
+	igt_assert_neq(ret, 0);
 
 	/* Both wrong. */
-	ret = gem_userptr(fd, (char *)ptr + 1, PAGE_SIZE - 1, 0, &handle);
-	igt_assert(ret != 0);
+	ret = __gem_userptr(fd, (char *)ptr + 1, PAGE_SIZE - 1, 0, userptr_flags, &handle);
+	igt_assert_neq(ret, 0);
 
 	/* Read-only not supported. */
-	ret = gem_userptr(fd, (char *)ptr, PAGE_SIZE, 1, &handle);
-	igt_assert(ret != 0);
+	ret = __gem_userptr(fd, (char *)ptr, PAGE_SIZE, 1, userptr_flags, &handle);
+	igt_assert_neq(ret, 0);
 
 	free(ptr);
 
 	return 0;
 }
 
-static int test_create_destroy(int fd)
+static int test_create_destroy(int fd, int time)
 {
-	void *ptr;
-	int ret;
+	struct timespec start, now;
 	uint32_t handle;
+	void *ptr;
+	int n;
 
-	igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE) == 0);
+	igt_fork_signal_helper();
 
-	ret = gem_userptr(fd, ptr, PAGE_SIZE, 0, &handle);
-	igt_assert(ret == 0);
+	clock_gettime(CLOCK_MONOTONIC, &start);
+	do {
+		for (n = 0; n < 1000; n++) {
+			igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE) == 0);
 
-	gem_close(fd, handle);
-	free(ptr);
+			do_or_die(__gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle));
+
+			gem_close(fd, handle);
+			free(ptr);
+		}
+
+		clock_gettime(CLOCK_MONOTONIC, &now);
+		now.tv_sec -= time;
+	} while (now.tv_sec < start.tv_sec ||
+		 (now.tv_sec == start.tv_sec && now.tv_nsec < start.tv_nsec));
+
+	igt_stop_signal_helper();
 
 	return 0;
 }
@@ -855,8 +869,8 @@ static int test_coherency(int fd, int count)
 	uint32_t start = 0;
 	int i, ret;
 
-	igt_require(intel_check_memory(count, sizeof(linear), CHECK_RAM));
 	igt_info("Using 2x%d 1MiB buffers\n", count);
+	intel_require_memory(2*count, sizeof(linear), CHECK_RAM);
 
 	ret = posix_memalign((void **)&memory, PAGE_SIZE, count*sizeof(linear));
 	igt_assert(ret == 0 && memory);
@@ -964,8 +978,6 @@ static int can_swap(void)
 	return 1;
 }
 
-#define min(a, b) ((a) < (b) ? (a) : (b))
-
 static void test_forking_evictions(int fd, int size, int count,
 			     unsigned flags)
 {
@@ -986,6 +998,13 @@ static void test_forking_evictions(int fd, int size, int count,
 	reset_handle_ptr();
 }
 
+static void test_mlocked_evictions(int fd, int size, int count)
+{
+	count = min(256, count/2);
+	mlocked_evictions(fd, &fault_ops, size, count);
+	reset_handle_ptr();
+}
+
 static void test_swapping_evictions(int fd, int size, int count)
 {
 	int trash_count;
@@ -1019,41 +1038,40 @@ static void test_overlap(int fd, int expected)
 
 	igt_assert(posix_memalign((void *)&ptr, PAGE_SIZE, PAGE_SIZE * 3) == 0);
 
-	ret = gem_userptr(fd, ptr + PAGE_SIZE, PAGE_SIZE, 0, &handle);
-	igt_assert(ret == 0);
+	gem_userptr(fd, ptr + PAGE_SIZE, PAGE_SIZE, 0, userptr_flags, &handle);
 
 	/* before, no overlap */
-	ret = gem_userptr(fd, ptr, PAGE_SIZE, 0, &handle2);
+	ret = __gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle2);
 	if (ret == 0)
 		gem_close(fd, handle2);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/* after, no overlap */
-	ret = gem_userptr(fd, ptr + PAGE_SIZE * 2, PAGE_SIZE, 0, &handle2);
+	ret = __gem_userptr(fd, ptr + PAGE_SIZE * 2, PAGE_SIZE, 0, userptr_flags, &handle2);
 	if (ret == 0)
 		gem_close(fd, handle2);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/* exactly overlapping */
-	ret = gem_userptr(fd, ptr + PAGE_SIZE, PAGE_SIZE, 0, &handle2);
+	ret = __gem_userptr(fd, ptr + PAGE_SIZE, PAGE_SIZE, 0, userptr_flags, &handle2);
 	if (ret == 0)
 		gem_close(fd, handle2);
 	igt_assert(ret == 0 || ret == expected);
 
 	/* start overlaps */
-	ret = gem_userptr(fd, ptr, PAGE_SIZE * 2, 0, &handle2);
+	ret = __gem_userptr(fd, ptr, PAGE_SIZE * 2, 0, userptr_flags, &handle2);
 	if (ret == 0)
 		gem_close(fd, handle2);
 	igt_assert(ret == 0 || ret == expected);
 
 	/* end overlaps */
-	ret = gem_userptr(fd, ptr + PAGE_SIZE, PAGE_SIZE * 2, 0, &handle2);
+	ret = __gem_userptr(fd, ptr + PAGE_SIZE, PAGE_SIZE * 2, 0, userptr_flags, &handle2);
 	if (ret == 0)
 		gem_close(fd, handle2);
 	igt_assert(ret == 0 || ret == expected);
 
 	/* subsumes */
-	ret = gem_userptr(fd, ptr, PAGE_SIZE * 3, 0, &handle2);
+	ret = __gem_userptr(fd, ptr, PAGE_SIZE * 3, 0, userptr_flags, &handle2);
 	if (ret == 0)
 		gem_close(fd, handle2);
 	igt_assert(ret == 0 || ret == expected);
@@ -1078,8 +1096,7 @@ static void test_unmap(int fd, int expected)
 	bo_ptr = (char *)ALIGN((unsigned long)ptr, PAGE_SIZE);
 
 	for (i = 0; i < num_obj; i++, bo_ptr += sizeof(linear)) {
-		ret = gem_userptr(fd, bo_ptr, sizeof(linear), 0, &bo[i]);
-		igt_assert(ret == 0);
+		gem_userptr(fd, bo_ptr, sizeof(linear), 0, userptr_flags, &bo[i]);
 	}
 
 	bo[num_obj] = create_bo(fd, 0);
@@ -1088,7 +1105,7 @@ static void test_unmap(int fd, int expected)
 		copy(fd, bo[num_obj], bo[i], 0);
 
 	ret = munmap(ptr, map_size);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	for (i = 0; i < num_obj; i++)
 		copy(fd, bo[num_obj], bo[i], expected);
@@ -1113,8 +1130,7 @@ static void test_unmap_after_close(int fd)
 	bo_ptr = (char *)ALIGN((unsigned long)ptr, PAGE_SIZE);
 
 	for (i = 0; i < num_obj; i++, bo_ptr += sizeof(linear)) {
-		ret = gem_userptr(fd, bo_ptr, sizeof(linear), 0, &bo[i]);
-		igt_assert(ret == 0);
+		gem_userptr(fd, bo_ptr, sizeof(linear), 0, userptr_flags, &bo[i]);
 	}
 
 	bo[num_obj] = create_bo(fd, 0);
@@ -1126,7 +1142,7 @@ static void test_unmap_after_close(int fd)
 		gem_close(fd, bo[i]);
 
 	ret = munmap(ptr, map_size);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 }
 
 static void test_unmap_cycles(int fd, int expected)
@@ -1137,6 +1153,8 @@ static void test_unmap_cycles(int fd, int expected)
 		test_unmap(fd, expected);
 }
 
+#define MM_STRESS_LOOPS 100000
+
 struct stress_thread_data {
 	unsigned int stop;
 	int exit_code;
@@ -1169,7 +1187,7 @@ static void test_stress_mm(int fd)
 {
 	int ret;
 	pthread_t t;
-	unsigned int loops = 100000;
+	unsigned int loops = MM_STRESS_LOOPS;
 	uint32_t handle;
 	void *ptr;
 	struct stress_thread_data stdata;
@@ -1179,11 +1197,10 @@ static void test_stress_mm(int fd)
 	igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE) == 0);
 
 	ret = pthread_create(&t, NULL, mm_stress_thread, &stdata);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	while (loops--) {
-		ret = gem_userptr(fd, ptr, PAGE_SIZE, 0, &handle);
-		igt_assert(ret == 0);
+		gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle);
 
 		gem_close(fd, handle);
 	}
@@ -1192,12 +1209,71 @@ static void test_stress_mm(int fd)
 
 	stdata.stop = 1;
 	ret = pthread_join(t, NULL);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
-	igt_assert(stdata.exit_code == 0);
+	igt_assert_eq(stdata.exit_code, 0);
 }
 
-unsigned int total_ram;
+struct userptr_close_thread_data {
+	int fd;
+	void *ptr;
+	bool overlap;
+	bool stop;
+	pthread_mutex_t mutex;
+};
+
+static void *mm_userptr_close_thread(void *data)
+{
+	struct userptr_close_thread_data *t = (struct userptr_close_thread_data *)data;
+	int num_handles = t->overlap ? 2 : 1;
+
+	uint32_t handle[num_handles];
+
+	/* Be pedantic and enforce the required memory barriers */
+	pthread_mutex_lock(&t->mutex);
+	while (!t->stop) {
+		pthread_mutex_unlock(&t->mutex);
+		for (int i = 0; i < num_handles; i++)
+			gem_userptr(t->fd, t->ptr, PAGE_SIZE, 0, userptr_flags, &handle[i]);
+		for (int i = 0; i < num_handles; i++)
+			gem_close(t->fd, handle[i]);
+		pthread_mutex_lock(&t->mutex);
+	}
+	pthread_mutex_unlock(&t->mutex);
+
+	return NULL;
+}
+
+static void test_invalidate_close_race(int fd, bool overlap)
+{
+	pthread_t t;
+	unsigned int loops = MM_STRESS_LOOPS;
+	struct userptr_close_thread_data t_data;
+
+	memset(&t_data, 0, sizeof(t_data));
+	t_data.fd = fd;
+	t_data.overlap = overlap;
+	igt_assert(posix_memalign(&t_data.ptr, PAGE_SIZE, PAGE_SIZE) == 0);
+	pthread_mutex_init(&t_data.mutex, NULL);
+
+	igt_assert(pthread_create(&t, NULL, mm_userptr_close_thread, &t_data) == 0);
+
+	while (loops--) {
+		mprotect(t_data.ptr, PAGE_SIZE, PROT_READ | PROT_WRITE | PROT_EXEC);
+		mprotect(t_data.ptr, PAGE_SIZE, PROT_READ | PROT_WRITE);
+	}
+
+	pthread_mutex_lock(&t_data.mutex);
+	t_data.stop = 1;
+	pthread_mutex_unlock(&t_data.mutex);
+
+	pthread_join(t, NULL);
+
+	pthread_mutex_destroy(&t_data.mutex);
+	free(t_data.ptr);
+}
+
+uint64_t total_ram;
 uint64_t aperture_size;
 int fd, count;
 
@@ -1206,14 +1282,12 @@ int main(int argc, char **argv)
 {
 	int size = sizeof(linear);
 
-	igt_skip_on_simulation();
-
 	igt_subtest_init(argc, argv);
 
 	igt_fixture {
 		int ret;
 
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
 		igt_assert(fd >= 0);
 
 		ret = has_userptr(fd);
@@ -1231,7 +1305,7 @@ int main(int argc, char **argv)
 			count = 2 * aperture_size / (1024*1024) / 3;
 
 		total_ram = intel_get_total_ram_mb();
-		igt_info("Total RAM is %u MiB\n", total_ram);
+		igt_info("Total RAM is %'llu MiB\n", (long long)total_ram);
 
 		if (count > total_ram * 3 / 4) {
 			count = intel_get_total_ram_mb() * 3 / 4;
@@ -1257,11 +1331,14 @@ int main(int argc, char **argv)
 	igt_subtest("forbidden-operations")
 		test_forbidden_ops(fd);
 
+	igt_subtest("relocations")
+		test_relocations(fd);
+
 	igt_info("Testing unsynchronized mappings...\n");
 	gem_userptr_test_unsynchronized();
 
 	igt_subtest("create-destroy-unsync")
-		test_create_destroy(fd);
+		test_create_destroy(fd, 5);
 
 	igt_subtest("unsync-overlap")
 		test_overlap(fd, 0);
@@ -1293,6 +1370,9 @@ int main(int argc, char **argv)
 		}
 	}
 
+	igt_subtest("mlocked-unsync-normal")
+		test_mlocked_evictions(fd, size, count);
+
 	igt_subtest("swapping-unsync-normal")
 		test_swapping_evictions(fd, size, count);
 
@@ -1314,6 +1394,9 @@ int main(int argc, char **argv)
 
 	igt_fork_signal_helper();
 
+	igt_subtest("mlocked-unsync-interruptible")
+		test_mlocked_evictions(fd, size, count);
+
 	igt_subtest("swapping-unsync-interruptible")
 		test_swapping_evictions(fd, size, count);
 
@@ -1352,7 +1435,7 @@ int main(int argc, char **argv)
 		test_process_exit(fd, PE_GTT_MAP | PE_BUSY);
 
 	igt_subtest("create-destroy-sync")
-		test_create_destroy(fd);
+		test_create_destroy(fd, 5);
 
 	igt_subtest("sync-overlap")
 		test_overlap(fd, EINVAL);
@@ -1367,7 +1450,13 @@ int main(int argc, char **argv)
 		test_unmap_after_close(fd);
 
 	igt_subtest("stress-mm")
-	        test_stress_mm(fd);
+		test_stress_mm(fd);
+
+	igt_subtest("stress-mm-invalidate-close")
+		test_invalidate_close_race(fd, false);
+
+	igt_subtest("stress-mm-invalidate-close-overlap")
+		test_invalidate_close_race(fd, true);
 
 	igt_subtest("coherency-sync")
 		test_coherency(fd, count);
@@ -1387,6 +1476,9 @@ int main(int argc, char **argv)
 		}
 	}
 
+	igt_subtest("mlocked-normal-sync")
+		test_mlocked_evictions(fd, size, count);
+
 	igt_subtest("swapping-normal-sync")
 		test_swapping_evictions(fd, size, count);
 
@@ -1408,6 +1500,9 @@ int main(int argc, char **argv)
 
 	igt_fork_signal_helper();
 
+	igt_subtest("mlocked-sync-interruptible")
+		test_mlocked_evictions(fd, size, count);
+
 	igt_subtest("swapping-sync-interruptible")
 		test_swapping_evictions(fd, size, count);
 
@@ -1426,6 +1521,4 @@ int main(int argc, char **argv)
 		test_access_control(fd);
 
 	igt_exit();
-
-	return 0;
 }
