@@ -25,6 +25,7 @@
  *
  */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -36,12 +37,9 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_chipset.h"
-#include "intel_io.h"
 #include "i830_reg.h"
-#include "igt_aux.h"
+
+IGT_TEST_DESCRIPTION("Check read/write syncpoints when switching rings.");
 
 #define LOCAL_I915_EXEC_VEBOX (4<<0)
 
@@ -154,7 +152,7 @@ igt_main
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
 
 		/* Test requires MI_FLUSH_DW and MI_COND_BATCH_BUFFER_END */
 		igt_require(intel_gen(intel_get_drm_devid(fd)) >= 6);
