@@ -22,17 +22,13 @@
  *
  */
 
+#include "igt.h"
 #include <errno.h>
 #include <limits.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "drmtest.h"
-#include "igt_debugfs.h"
-#include "igt_kms.h"
-#include "intel_chipset.h"
-#include "igt_aux.h"
 
 IGT_TEST_DESCRIPTION(
    "Use the display CRC support to validate cursor plane functionality. "
@@ -59,12 +55,17 @@ typedef struct {
 	igt_crc_t ref_crc;
 	int left, right, top, bottom;
 	int screenw, screenh;
+	int refresh;
 	int curw, curh; /* cursor size */
 	int cursor_max_w, cursor_max_h;
 	igt_pipe_crc_t *pipe_crc;
 	uint32_t devid;
+	unsigned flags;
 } data_t;
 
+#define TEST_DPMS (1<<0)
+#define TEST_SUSPEND (1<<1)
+
 static void draw_cursor(cairo_t *cr, int x, int y, int cw, int ch)
 {
 	int wl, wr, ht, hb;
@@ -124,19 +125,40 @@ static void do_single_test(data_t *data, int x, int y)
 	igt_display_commit(display);
 	igt_wait_for_vblank(data->drm_fd, data->pipe);
 	igt_pipe_crc_collect_crc(pipe_crc, &crc);
+
+	if (data->flags & (TEST_DPMS | TEST_SUSPEND)) {
+		igt_crc_t crc_after;
+
+		if (data->flags & TEST_DPMS) {
+			igt_debug("dpms off/on cycle\n");
+			kmstest_set_connector_dpms(data->drm_fd,
+						   data->output->config.connector,
+						   DRM_MODE_DPMS_OFF);
+			kmstest_set_connector_dpms(data->drm_fd,
+						   data->output->config.connector,
+						   DRM_MODE_DPMS_ON);
+		}
+
+		if (data->flags & TEST_SUSPEND)
+			igt_system_suspend_autoresume();
+
+		igt_pipe_crc_collect_crc(pipe_crc, &crc_after);
+		igt_assert_crc_equal(&crc, &crc_after);
+	}
+
 	cursor_disable(data);
 	igt_display_commit(display);
 
 	/* Now render the same in software and collect crc */
 	draw_cursor(cr, x, y, data->curw, data->curh);
 	igt_display_commit(display);
+
 	igt_wait_for_vblank(data->drm_fd, data->pipe);
 	igt_pipe_crc_collect_crc(pipe_crc, &ref_crc);
-	/* Clear screen afterwards */
-	igt_paint_color(cr, 0, 0, data->screenw, data->screenh,
-			0.0, 0.0, 0.0);
+	igt_assert_crc_equal(&crc, &ref_crc);
 
-	igt_assert(igt_crc_equal(&crc, &ref_crc));
+	/* Clear screen afterwards */
+	igt_paint_color(cr, 0, 0, data->screenw, data->screenh, 0.0, 0.0, 0.0);
 }
 
 static void do_fail_test(data_t *data, int x, int y, int expect)
@@ -247,10 +269,12 @@ static void test_crc_sliding(data_t *data)
 
 static void test_crc_random(data_t *data)
 {
-	int i;
+	int i, max;
+
+	max = data->flags & (TEST_DPMS | TEST_SUSPEND) ? 2 : 50;
 
 	/* Random cursor placement */
-	for (i = 0; i < 50; i++) {
+	for (i = 0; i < max; i++) {
 		int x = rand() % (data->screenw + data->curw * 2) - data->curw;
 		int y = rand() % (data->screenh + data->curh * 2) - data->curh;
 		do_single_test(data, x, y);
@@ -304,6 +328,7 @@ static bool prepare_crtc(data_t *data, igt_output_t *output,
 	data->screenh = mode->vdisplay;
 	data->curw = cursor_w;
 	data->curh = cursor_h;
+	data->refresh = mode->vrefresh;
 
 	/* make sure cursor is disabled */
 	cursor_disable(data);
@@ -452,10 +477,46 @@ static void test_cursor_size(data_t *data)
 		/* Clear screen afterwards */
 		igt_paint_color(cr, 0, 0, data->screenw, data->screenh,
 				0.0, 0.0, 0.0);
-		igt_assert(igt_crc_equal(&crc[i], &ref_crc));
+		igt_assert_crc_equal(&crc[i], &ref_crc);
 	}
 }
 
+static void test_rapid_movement(data_t *data)
+{
+	struct timeval start, end, delta;
+	int x = 0, y = 0;
+	long usec;
+	int crtc_id = data->output->config.crtc->crtc_id;
+
+	igt_assert_eq(drmModeSetCursor(data->drm_fd, crtc_id,
+			       data->fb.gem_handle, data->curw, data->curh), 0);
+
+	gettimeofday(&start, NULL);
+	for ( ; x < 100; x++)
+		igt_assert_eq(drmModeMoveCursor(data->drm_fd, crtc_id, x, y), 0);
+	for ( ; y < 100; y++)
+		igt_assert_eq(drmModeMoveCursor(data->drm_fd, crtc_id, x, y), 0);
+	for ( ; x > 0; x--)
+		igt_assert_eq(drmModeMoveCursor(data->drm_fd, crtc_id, x, y), 0);
+	for ( ; y > 0; y--)
+		igt_assert_eq(drmModeMoveCursor(data->drm_fd, crtc_id, x, y), 0);
+	gettimeofday(&end, NULL);
+
+	/*
+	 * We've done 400 cursor updates now.  If we're being throttled to
+	 * vblank, then that would take roughly 400/refresh seconds.  If the
+	 * elapsed time is greater than 90% of that value, we'll consider it
+	 * a failure (since cursor updates shouldn't be throttled).
+	 */
+	timersub(&end, &start, &delta);
+	usec = delta.tv_usec + 1000000 * delta.tv_sec;
+	igt_assert_lt(usec, 0.9 * 400 * 1000000 / data->refresh);
+
+	igt_assert_eq(drmModeSetCursor(data->drm_fd, crtc_id,
+			       0, data->curw, data->curh), 0);
+
+}
+
 static void run_test_generic(data_t *data)
 {
 	int cursor_size;
@@ -475,6 +536,21 @@ static void run_test_generic(data_t *data)
 			run_test(data, test_crc_sliding, w, h);
 		igt_subtest_f("cursor-%dx%d-random", w, h)
 			run_test(data, test_crc_random, w, h);
+		igt_subtest_f("cursor-%dx%d-dpms", w, h) {
+			data->flags = TEST_DPMS;
+			run_test(data, test_crc_random, w, h);
+			data->flags = 0;
+		}
+
+		igt_subtest_f("cursor-%dx%d-suspend", w, h) {
+			data->flags = TEST_SUSPEND;
+			run_test(data, test_crc_random, w, h);
+			data->flags = 0;
+		}
+
+		igt_subtest_f("cursor-%dx%d-rapid-movement", w, h) {
+			run_test(data, test_rapid_movement, w, h);
+		}
 
 		igt_fixture
 			igt_remove_fb(data->drm_fd, &data->fb);
@@ -522,7 +598,7 @@ igt_main
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		data.drm_fd = drm_open_any_master();
+		data.drm_fd = drm_open_driver_master(DRIVER_INTEL);
 
 		data.devid = intel_get_drm_devid(data.drm_fd);
 
