@@ -21,17 +21,13 @@
  * IN THE SOFTWARE.
  */
 
+#include "igt.h"
 #include <errno.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
 
-#include "drmtest.h"
-#include "igt_debugfs.h"
-#include "igt_kms.h"
-#include "intel_chipset.h"
-#include "ioctl_wrappers.h"
 
 typedef struct {
 	int drm_fd;
@@ -127,7 +123,7 @@ static void wait_for_flip(data_t *data, uint32_t flip_handle)
 		if (ret < 0 && errno == EINTR)
 			continue;
 
-		igt_assert(ret >= 0);
+		igt_assert_lte(0, ret);
 
 		do_or_die(drmHandleEvent(data->drm_fd, &evctx));
 	}
@@ -218,17 +214,17 @@ test_plane(data_t *data, igt_output_t *output, enum pipe pipe, enum igt_plane pl
 	mode = igt_output_get_mode(output);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 			    DRM_FORMAT_XRGB8888,
-			    I915_TILING_NONE,
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    1.0, 0.0, 0.0,
 			    &red_fb);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 			    DRM_FORMAT_XRGB8888,
-			    I915_TILING_NONE,
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    0.0, 1.0, 0.0,
 			    &green_fb);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 			    DRM_FORMAT_XRGB8888,
-			    I915_TILING_NONE,
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    0.0, 0.0, 1.0,
 			    &blue_fb);
 
@@ -247,21 +243,6 @@ test_plane(data_t *data, igt_output_t *output, enum pipe pipe, enum igt_plane pl
 	if (data->pipe_crc)
 		igt_pipe_crc_free(data->pipe_crc);
 	data->pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
-	if (!data->pipe_crc) {
-		igt_info("auto crc not supported on this connector with crtc %i\n",
-			 pipe);
-
-		igt_plane_set_fb(primary, NULL);
-		igt_plane_set_fb(sprite, NULL);
-		igt_output_set_pipe(output, PIPE_ANY);
-		igt_display_commit(&data->display);
-
-		igt_remove_fb(data->drm_fd, &red_fb);
-		igt_remove_fb(data->drm_fd, &green_fb);
-		igt_remove_fb(data->drm_fd, &blue_fb);
-
-		return false;
-	}
 
 	/* set red fb and grab reference crc */
 	igt_plane_set_fb(primary, &red_fb);
@@ -271,14 +252,14 @@ test_plane(data_t *data, igt_output_t *output, enum pipe pipe, enum igt_plane pl
 	ret = drmModeSetCrtc(data->drm_fd, output->config.crtc->crtc_id,
 			     blue_fb.fb_id, 0, 0, &output->id, 1,
 			     mode);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	make_gpu_busy(data, blue_fb.gem_handle);
 
 	data->flip_done = false;
 	ret = drmModePageFlip(data->drm_fd, output->config.crtc->crtc_id,
 			      blue_fb.fb_id, DRM_MODE_PAGE_FLIP_EVENT, data);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/*
 	 * Toggle a fullscreen sprite on and back off. This will result
@@ -294,14 +275,14 @@ test_plane(data_t *data, igt_output_t *output, enum pipe pipe, enum igt_plane pl
 			      green_fb.fb_id, 0,
 			      0, 0, mode->hdisplay, mode->vdisplay,
 			      0, 0, mode->hdisplay << 16, mode->vdisplay << 16);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 	ret = drmModeSetPlane(data->drm_fd,
 			      sprite->drm_plane->plane_id,
 			      output->config.crtc->crtc_id,
 			      0, 0,
 			      0, 0, 0, 0,
 			      0, 0, 0, 0);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/*
 	 * Set primary plane to red fb. This should wait for the CS flip
@@ -313,7 +294,7 @@ test_plane(data_t *data, igt_output_t *output, enum pipe pipe, enum igt_plane pl
 	ret = drmModeSetCrtc(data->drm_fd, output->config.crtc->crtc_id,
 			     red_fb.fb_id, 0, 0, &output->id, 1,
 			     mode);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/* Make sure the flip has been executed */
 	wait_for_flip(data, blue_fb.gem_handle);
@@ -334,7 +315,7 @@ test_plane(data_t *data, igt_output_t *output, enum pipe pipe, enum igt_plane pl
 	igt_output_set_pipe(output, PIPE_ANY);
 	igt_display_commit(&data->display);
 
-	igt_assert(igt_crc_equal(&ref_crc, &crc));
+	igt_assert_crc_equal(&ref_crc, &crc);
 
 	return true;
 }
@@ -383,17 +364,17 @@ test_crtc(data_t *data, igt_output_t *output, enum pipe pipe)
 	mode = igt_output_get_mode(output);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay+1,
 			    DRM_FORMAT_XRGB8888,
-			    I915_TILING_NONE,
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    1.0, 0.0, 0.0,
 			    &red_fb);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay+1,
 			    DRM_FORMAT_XRGB8888,
-			    I915_TILING_NONE,
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    0.0, 0.0, 1.0,
 			    &blue_fb);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay+1,
 			    DRM_FORMAT_XRGB8888,
-			    I915_TILING_NONE,
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    0.0, 1.0, 0.0,
 			    &green_fb);
 
@@ -410,20 +391,6 @@ test_crtc(data_t *data, igt_output_t *output, enum pipe pipe)
 	if (data->pipe_crc)
 		igt_pipe_crc_free(data->pipe_crc);
 	data->pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
-	if (!data->pipe_crc) {
-		igt_info("auto crc not supported on this connector with crtc %i\n",
-			 pipe);
-
-		igt_plane_set_fb(primary, NULL);
-		igt_output_set_pipe(output, PIPE_ANY);
-		igt_display_commit(&data->display);
-
-		igt_remove_fb(data->drm_fd, &red_fb);
-		igt_remove_fb(data->drm_fd, &green_fb);
-		igt_remove_fb(data->drm_fd, &blue_fb);
-
-		return false;
-	}
 
 	/* set red fb and grab reference crc */
 	igt_plane_set_fb(primary, &red_fb);
@@ -439,7 +406,7 @@ test_crtc(data_t *data, igt_output_t *output, enum pipe pipe)
 	ret = drmModeSetCrtc(data->drm_fd, output->config.crtc->crtc_id,
 			     green_fb.fb_id, 0, 1, &output->id, 1,
 			     mode);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/*
 	 * Make it more likely that the CS flip has been submitted into the
@@ -453,7 +420,7 @@ test_crtc(data_t *data, igt_output_t *output, enum pipe pipe)
 	ret = drmModeSetCrtc(data->drm_fd, output->config.crtc->crtc_id,
 			     green_fb.fb_id, 0, 0, &output->id, 1,
 			     mode);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	make_gpu_busy(data, blue_fb.gem_handle);
 
@@ -464,7 +431,7 @@ test_crtc(data_t *data, igt_output_t *output, enum pipe pipe)
 	data->flip_done = false;
 	ret = drmModePageFlip(data->drm_fd, output->config.crtc->crtc_id,
 			      blue_fb.fb_id, DRM_MODE_PAGE_FLIP_EVENT, data);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/*
 	 * Set primary plane to red fb. This should wait for the CS flip
@@ -476,7 +443,7 @@ test_crtc(data_t *data, igt_output_t *output, enum pipe pipe)
 	ret = drmModeSetCrtc(data->drm_fd, output->config.crtc->crtc_id,
 			     red_fb.fb_id, 0, 0, &output->id, 1,
 			     mode);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	/* Make sure the flip has been executed */
 	wait_for_flip(data, blue_fb.gem_handle);
@@ -497,37 +464,43 @@ test_crtc(data_t *data, igt_output_t *output, enum pipe pipe)
 	igt_output_set_pipe(output, PIPE_ANY);
 	igt_display_commit(&data->display);
 
-	igt_assert(igt_crc_equal(&ref_crc, &crc));
+	igt_assert_crc_equal(&ref_crc, &crc);
 
 	return true;
 }
 
 static void
-run_plane_test_for_pipe(data_t *data, enum pipe pipe)
+run_plane_test(data_t *data)
 {
 	igt_output_t *output;
 	enum igt_plane plane = 1; /* testing with one sprite is enough */
 	int valid_tests = 0;
-
-	igt_require(data->display.pipes[pipe].n_planes > 2);
+	enum pipe pipe;
 
 	for_each_connected_output(&data->display, output) {
-		if (test_plane(data, output, pipe, plane))
-			valid_tests++;
+		for_each_pipe(&data->display, pipe) {
+			igt_require(data->display.pipes[pipe].n_planes > 2);
+
+			if (test_plane(data, output, pipe, plane))
+				valid_tests++;
+		}
 	}
 
 	igt_require_f(valid_tests, "no valid crtc/connector combinations found\n");
 }
 
 static void
-run_crtc_test_for_pipe(data_t *data, enum pipe pipe)
+run_crtc_test(data_t *data)
 {
 	igt_output_t *output;
 	int valid_tests = 0;
+	enum pipe pipe;
 
 	for_each_connected_output(&data->display, output) {
-		if (test_crtc(data, output, pipe))
-			valid_tests++;
+		for_each_pipe(&data->display, pipe) {
+			if (test_crtc(data, output, pipe))
+				valid_tests++;
+		}
 	}
 
 	igt_require_f(valid_tests, "no valid crtc/connector combinations found\n");
@@ -537,12 +510,10 @@ static data_t data;
 
 igt_main
 {
-	int pipe;
-
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		data.drm_fd = drm_open_any();
+		data.drm_fd = drm_open_driver_master(DRIVER_INTEL);
 
 		kmstest_set_vt_graphics_mode();
 
@@ -560,15 +531,11 @@ igt_main
 		gem_set_tiling(data.drm_fd, data.busy_bo->handle, 0, 4096);
 	}
 
-	igt_subtest_f("setplane_vs_cs_flip") {
-		for (pipe = 0; pipe < data.display.n_pipes; pipe++)
-			run_plane_test_for_pipe(&data, pipe);
-	}
+	igt_subtest_f("setplane_vs_cs_flip")
+		run_plane_test(&data);
 
-	igt_subtest_f("setcrtc_vs_cs_flip") {
-		for (pipe = 0; pipe < data.display.n_pipes; pipe++)
-			run_crtc_test_for_pipe(&data, pipe);
-	}
+	igt_subtest_f("setcrtc_vs_cs_flip")
+		run_crtc_test(&data);
 
 	igt_fixture {
 		drm_intel_bo_unreference(data.busy_bo);
