@@ -199,68 +199,27 @@ static void fill_render(data_t *data, uint32_t handle, unsigned char color)
 
 static bool psr_possible(data_t *data)
 {
-	FILE *file;
-	char buf[4096];
-	int ret;
+	char buf[512];
 
-	if (running_with_psr_disabled)
-		return true;
+	igt_debugfs_read("i915_edp_psr_status", buf);
 
-	file = igt_debugfs_fopen("i915_edp_psr_status", "r");
-	igt_require(file);
-
-	/* First dump the entire file into the debug log for later analysis
-	 * if required.
-	 */
-	ret = fread(buf, 1, 4095, file);
-	igt_require(ret > 0);
-	buf[ret] = '\0';
-	igt_debug("i915_edp_psr_status:\n%s", buf);
-	fseek(file, 0, SEEK_SET);
-
-	/* Now check that we have all the preconditions required for PSR */
-	ret = fscanf(file, "Sink_Support: %s\n", buf);
-	igt_require_f(ret == 1 && strcmp(buf, "yes") == 0,
-		      "Sink_Support: %s\n", buf);
-
-	fclose(file);
-	return true;
+	return running_with_psr_disabled ||
+		strstr(buf, "Sink_Support: yes\n");
 }
 
 static bool psr_active(data_t *data)
 {
-	int ret;
-	FILE *file;
-	char str[4];
+	char buf[512];
 
-	if (running_with_psr_disabled)
-		return true;
+	igt_debugfs_read("i915_edp_psr_status", buf);
 
-	file = igt_debugfs_fopen("i915_edp_psr_status", "r");
-	igt_require(file);
-
-	ret = fscanf(file, "Sink_Support: %s\n", str);
-	igt_assert_neq(ret, 0);
-	ret = fscanf(file, "Source_OK: %s\n", str);
-	igt_assert_neq(ret, 0);
-	ret = fscanf(file, "Enabled: %s\n", str);
-	igt_assert_neq(ret, 0);
-	ret = fscanf(file, "Active: %s\n", str);
-	igt_assert_neq(ret, 0);
-	ret = fscanf(file, "Busy frontbuffer bits: %s\n", str);
-	igt_assert_neq(ret, 0);
-	ret = fscanf(file, "Re-enable work scheduled: %s\n", str);
-	igt_assert_neq(ret, 0);
-	ret = fscanf(file, "HW Enabled & Active bit: %s\n", str);
-	igt_assert_neq(ret, 0);
-
-	fclose(file);
-	return strcmp(str, "yes") == 0;
+	return running_with_psr_disabled ||
+		strstr(buf, "HW Enabled & Active bit: yes\n");
 }
 
 static bool wait_psr_entry(data_t *data)
 {
-	int timeout = 10;
+	int timeout = 5;
 	while (timeout--) {
 		if (psr_active(data))
 			return true;
@@ -557,7 +516,8 @@ int main(int argc, char *argv[])
 		kmstest_set_vt_graphics_mode();
 		data.devid = intel_get_drm_devid(data.drm_fd);
 
-		igt_set_module_param_int("enable_psr", 1);
+		igt_set_module_param_int("enable_psr", running_with_psr_disabled ?
+					 0 : 1);
 
 		igt_skip_on(!psr_possible(&data));
 
@@ -568,6 +528,11 @@ int main(int argc, char *argv[])
 		display_init(&data);
 	}
 
+	igt_subtest("psr_basic") {
+		setup_test_plane(&data);
+		igt_assert(wait_psr_entry(&data));
+	}
+
 	for (op = PAGE_FLIP; op <= RENDER; op++) {
 		igt_subtest_f("primary_%s", op_str(op)) {
 			data.test_plane = PRIMARY;
@@ -625,6 +590,30 @@ int main(int argc, char *argv[])
 		test_cleanup(&data);
 	}
 
+	igt_subtest_f("suspend_psr_active") {
+		data.test_plane = PRIMARY;
+		data.op = PAGE_FLIP;
+		setup_test_plane(&data);
+		igt_assert(wait_psr_entry(&data));
+
+		igt_system_suspend_autoresume();
+
+		run_test(&data);
+		test_cleanup(&data);
+	}
+
+	igt_subtest_f("suspend_psr_exit") {
+		data.test_plane = CURSOR;
+		data.op = PLANE_ONOFF;
+		setup_test_plane(&data);
+
+		igt_system_suspend_autoresume();
+
+		igt_assert(wait_psr_entry(&data));
+		run_test(&data);
+		test_cleanup(&data);
+	}
+
 	igt_fixture {
 		drm_intel_bufmgr_destroy(data.bufmgr);
 		display_fini(&data);
