@@ -49,6 +49,7 @@
 #include "config.h"
 #endif
 
+#include "igt.h"
 #include <cairo.h>
 #include <errno.h>
 #include <getopt.h>
@@ -64,11 +65,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
 #include "testdisplay.h"
-#include "igt_kms.h"
-#include "igt_core.h"
 
 #include <stdlib.h>
 #include <signal.h>
@@ -135,7 +132,8 @@ static void dump_connectors_fd(int drmfd)
 	for (i = 0; i < mode_resources->count_connectors; i++) {
 		drmModeConnector *connector;
 
-		connector = drmModeGetConnector(drmfd, mode_resources->connectors[i]);
+		connector = drmModeGetConnectorCurrent(drmfd,
+						       mode_resources->connectors[i]);
 		if (!connector) {
 			igt_warn("could not get connector %i: %s\n", mode_resources->connectors[i], strerror(errno));
 			continue;
@@ -193,12 +191,19 @@ static void dump_info(void)
 
 static void connector_find_preferred_mode(uint32_t connector_id,
 					  unsigned long crtc_idx_mask,
-					  int mode_num, struct connector *c)
+					  int mode_num, struct connector *c,
+					  bool probe)
 {
 	struct kmstest_connector_config config;
+	bool ret;
 
-	if (!kmstest_get_connector_config(drm_fd, connector_id, crtc_idx_mask,
-					  &config)) {
+	if (probe)
+		ret = kmstest_probe_connector_config(drm_fd, connector_id,
+						     crtc_idx_mask, &config);
+	else
+		ret = kmstest_get_connector_config(drm_fd, connector_id,
+						   crtc_idx_mask, &config);
+	if (!ret) {
 		c->mode_valid = 0;
 		return;
 	}
@@ -224,9 +229,8 @@ paint_color_key(struct igt_fb *fb_info)
 	int i, j;
 	uint32_t *fb_ptr;
 
-	fb_ptr = gem_mmap(drm_fd, fb_info->gem_handle,
-			  fb_info->size, PROT_READ | PROT_WRITE);
-	igt_assert(fb_ptr);
+	fb_ptr = gem_mmap__gtt(drm_fd, fb_info->gem_handle, fb_info->size,
+			       PROT_READ | PROT_WRITE);
 
 	for (i = crtc_y; i < crtc_y + crtc_h; i++)
 		for (j = crtc_x; j < crtc_x + crtc_w; j++) {
@@ -262,8 +266,6 @@ static void paint_output_info(struct connector *c, struct igt_fb *fb)
 	double max_width;
 	int i;
 
-	igt_paint_test_pattern(cr, l_width, l_height);
-
 	cairo_move_to(cr, l_width / 2, l_height / 2);
 
 	/* Print connector and mode name */
@@ -356,9 +358,9 @@ set_mode(struct connector *c)
 		width = c->mode.hdisplay;
 		height = c->mode.vdisplay;
 
-		fb_id = igt_create_fb(drm_fd, width, height,
-				      igt_bpp_depth_to_drm_format(bpp, depth),
-				      tiling, &fb_info[current_fb]);
+		fb_id = igt_create_pattern_fb(drm_fd, width, height,
+					      igt_bpp_depth_to_drm_format(bpp, depth),
+					      tiling, &fb_info[current_fb]);
 		paint_output_info(c, &fb_info[current_fb]);
 		paint_color_key(&fb_info[current_fb]);
 
@@ -367,6 +369,7 @@ set_mode(struct connector *c)
 		if (drmModeSetCrtc(drm_fd, c->crtc, fb_id, 0, 0,
 				   &c->id, 1, &c->mode)) {
 			igt_warn("failed to set mode (%dx%d@%dHz): %s\n", width, height, c->mode.vrefresh, strerror(errno));
+			igt_remove_fb(drm_fd, &fb_info[current_fb]);
 			continue;
 		}
 
@@ -390,7 +393,7 @@ set_mode(struct connector *c)
 		}
 	}
 
-	if (test_all_modes)
+	if (test_all_modes && old_fb != -1)
 		igt_remove_fb(drm_fd, &fb_info[old_fb]);
 
 	drmModeFreeEncoder(c->encoder);
@@ -460,7 +463,7 @@ set_stereo_mode(struct connector *c)
  * Each connector has a corresponding encoder, except in the SDVO case
  * where an encoder may have multiple connectors.
  */
-int update_display(void)
+int update_display(bool probe)
 {
 	struct connector *connectors;
 	int c;
@@ -492,7 +495,7 @@ int update_display(void)
 			connector_find_preferred_mode(connector->id,
 						      crtc_idx_mask,
 						      specified_mode_num,
-						      connector);
+						      connector, probe);
 			if (!connector->mode_valid)
 				continue;
 
@@ -517,7 +520,7 @@ int update_display(void)
 			connector_find_preferred_mode(connector->id,
 						      -1UL,
 						      specified_mode_num,
-						      connector);
+						      connector, probe);
 			if (!connector->mode_valid)
 				continue;
 
@@ -555,7 +558,7 @@ static void __attribute__((noreturn)) usage(char *name, char opt)
 
 #define dump_resource(res) if (res) dump_##res()
 
-static void cleanup_and_exit(int ret)
+static void __attribute__((noreturn)) cleanup_and_exit(int ret)
 {
 	close(drm_fd);
 	exit(ret);
@@ -727,7 +730,7 @@ int main(int argc, char **argv)
 	    specified_mode_num == -1 && !test_stereo_modes)
 		test_all_modes = 1;
 
-	drm_fd = drm_open_any();
+	drm_fd = drm_open_driver(DRIVER_INTEL);
 
 	if (test_stereo_modes &&
 	    drmSetClientCap(drm_fd, DRM_CLIENT_CAP_STEREO_3D, 1) < 0) {
@@ -769,7 +772,7 @@ int main(int argc, char **argv)
 
 	ret = 0;
 
-	if (!update_display()) {
+	if (!update_display(false)) {
 		ret = 1;
 		goto out_stdio;
 	}
