@@ -65,37 +65,39 @@ uint8_t *VBIOS;
 
 #define YESNO(val) ((val) ? "yes" : "no")
 
+/* This is not for mapping to memory layout. */
 struct bdb_block {
 	uint8_t id;
-	uint16_t size;
-	void *data;
+	uint32_t size;
+	const void *data;
 };
 
-static const char * const seq_name[] = {
-	"UNDEFINED",
-	"MIPI_SEQ_ASSERT_RESET",
-	"MIPI_SEQ_INIT_OTP",
-	"MIPI_SEQ_DISPLAY_ON",
-	"MIPI_SEQ_DISPLAY_OFF",
-	"MIPI_SEQ_DEASSERT_RESET",
-};
-
-struct bdb_header *bdb;
-struct bdb_lvds_lfp_data_ptrs *lvds_lfp_data_ptrs;
+const struct bdb_lvds_lfp_data_ptrs *lvds_lfp_data_ptrs;
 static int tv_present;
 static int lvds_present;
 static int panel_type;
 
-static struct bdb_block *find_section(int section_id, int length)
+/* Get BDB block size given a pointer to Block ID. */
+static uint32_t _get_blocksize(const uint8_t *block_base)
+{
+	/* The MIPI Sequence Block v3+ has a separate size field. */
+	if (*block_base == BDB_MIPI_SEQUENCE && *(block_base + 3) >= 3)
+		return *((const uint32_t *)(block_base + 4));
+	else
+		return *((const uint16_t *)(block_base + 1));
+}
+
+static struct bdb_block *find_section(const struct bdb_header *bdb,
+				      int section_id, int length)
 {
 	struct bdb_block *block;
-	unsigned char *base = (unsigned char *)bdb;
-	int idx = 0;
-	uint16_t total, current_size;
+	const uint8_t *base = (const uint8_t *)bdb;
+	int index = 0;
+	uint32_t total, current_size;
 	unsigned char current_id;
 
 	/* skip to first section */
-	idx += bdb->header_size;
+	index += bdb->header_size;
 	total = bdb->bdb_size;
 	if (total > length)
 		total = length;
@@ -107,29 +109,32 @@ static struct bdb_block *find_section(int section_id, int length)
 	}
 
 	/* walk the sections looking for section_id */
-	while (idx + 3 < total) {
-		current_id = *(base + idx);
-		current_size = *(uint16_t *)(base + idx + 1);
-		if (idx + current_size > total)
+	while (index + 3 < total) {
+		current_id = *(base + index);
+		current_size = _get_blocksize(base + index);
+		index += 3;
+
+		if (index + current_size > total)
 			return NULL;
 
 		if (current_id == section_id) {
 			block->id = current_id;
 			block->size = current_size;
-			block->data = base + idx + 3;
+			block->data = base + index;
 			return block;
 		}
 
-		idx += current_size + 3;
+		index += current_size;
 	}
 
 	free(block);
 	return NULL;
 }
 
-static void dump_general_features(const struct bdb_block *block)
+static void dump_general_features(const struct bdb_header *bdb,
+				  const struct bdb_block *block)
 {
-	struct bdb_general_features *features = block->data;
+	const struct bdb_general_features *features = block->data;
 
 	printf("\tPanel fitting: ");
 	switch (features->panel_fitting) {
@@ -179,10 +184,11 @@ static void dump_general_features(const struct bdb_block *block)
 	lvds_present = 1;	/* should be based on IS_MOBILE() */
 }
 
-static void dump_backlight_info(const struct bdb_block *block)
+static void dump_backlight_info(const struct bdb_header *bdb,
+				const struct bdb_block *block)
 {
-	struct bdb_lvds_backlight *backlight = block->data;
-	struct blc_struct *blc;
+	const struct bdb_lvds_backlight *backlight = block->data;
+	const struct blc_struct *blc;
 
 	if (sizeof(struct blc_struct) != backlight->blcstruct_size) {
 		printf("\tBacklight struct sizes don't match (expected %zu, got %u), skipping\n",
@@ -361,7 +367,8 @@ static const char *efp_conn(uint8_t type)
 
 
 
-static void dump_child_device(struct child_device_config *child)
+static void dump_child_device(const struct bdb_header *bdb,
+			      struct child_device_config *child)
 {
 	char child_id[11];
 
@@ -399,9 +406,10 @@ static void dump_child_device(struct child_device_config *child)
 	}
 }
 
-static void dump_general_definitions(const struct bdb_block *block)
+static void dump_general_definitions(const struct bdb_header *bdb,
+				     const struct bdb_block *block)
 {
-	struct bdb_general_definitions *defs = block->data;
+	const struct bdb_general_definitions *defs = block->data;
 	int i;
 	int child_device_num;
 
@@ -418,13 +426,14 @@ static void dump_general_definitions(const struct bdb_block *block)
 	child_device_num = (block->size - sizeof(*defs)) /
 		defs->child_dev_size;
 	for (i = 0; i < child_device_num; i++)
-		dump_child_device((void*)&defs->devices[i * defs->child_dev_size]);
+		dump_child_device(bdb,(void*)&defs->devices[i * defs->child_dev_size]);
 }
 
-static void dump_child_devices(const struct bdb_block *block)
+static void dump_child_devices(const struct bdb_header *bdb,
+			       const struct bdb_block *block)
 {
-	struct bdb_child_devices *child_devs = block->data;
-	struct child_device_config *child;
+	const struct bdb_child_devices *child_devs = block->data;
+	const struct child_device_config *child;
 	int i;
 
 	for (i = 0; i < DEVICE_CHILD_SIZE; i++) {
@@ -444,9 +453,10 @@ static void dump_child_devices(const struct bdb_block *block)
 	}
 }
 
-static void dump_lvds_options(const struct bdb_block *block)
+static void dump_lvds_options(const struct bdb_header *bdb,
+			      const struct bdb_block *block)
 {
-	struct bdb_lvds_options *options = block->data;
+	const struct bdb_lvds_options *options = block->data;
 
 	panel_type = options->panel_type;
 	printf("\tPanel type: %d\n", panel_type);
@@ -460,9 +470,10 @@ static void dump_lvds_options(const struct bdb_block *block)
 	printf("\tPFIT mode: %d\n", options->pfit_mode);
 }
 
-static void dump_lvds_ptr_data(const struct bdb_block *block)
+static void dump_lvds_ptr_data(const struct bdb_header *bdb,
+			       const struct bdb_block *block)
 {
-	struct bdb_lvds_lfp_data_ptrs *ptrs = block->data;
+	const struct bdb_lvds_lfp_data_ptrs *ptrs = block->data;
 
 	printf("\tNumber of entries: %d\n", ptrs->lvds_entries);
 
@@ -470,10 +481,11 @@ static void dump_lvds_ptr_data(const struct bdb_block *block)
 	lvds_lfp_data_ptrs = ptrs;
 }
 
-static void dump_lvds_data(const struct bdb_block *block)
+static void dump_lvds_data(const struct bdb_header *bdb,
+			   const struct bdb_block *block)
 {
-	struct bdb_lvds_lfp_data *lvds_data = block->data;
-	struct bdb_lvds_lfp_data_ptrs *ptrs = lvds_lfp_data_ptrs;
+	const struct bdb_lvds_lfp_data *lvds_data = block->data;
+	const struct bdb_lvds_lfp_data_ptrs *ptrs = lvds_lfp_data_ptrs;
 	int num_entries;
 	int i;
 	int hdisplay, hsyncstart, hsyncend, htotal;
@@ -497,11 +509,11 @@ static void dump_lvds_data(const struct bdb_block *block)
 	       num_entries);
 
 	for (i = 0; i < num_entries; i++) {
-		uint8_t *lfp_data_ptr =
-		    (uint8_t *) lvds_data->data + lfp_data_size * i;
-		uint8_t *timing_data = lfp_data_ptr + dvo_offset;
-		struct bdb_lvds_lfp_data_entry *lfp_data =
-		    (struct bdb_lvds_lfp_data_entry *)lfp_data_ptr;
+		const uint8_t *lfp_data_ptr =
+		    (const uint8_t *) lvds_data->data + lfp_data_size * i;
+		const uint8_t *timing_data = lfp_data_ptr + dvo_offset;
+		const struct bdb_lvds_lfp_data_entry *lfp_data =
+		    (const struct bdb_lvds_lfp_data_entry *)lfp_data_ptr;
 		char marker;
 
 		if (i == panel_type)
@@ -542,9 +554,10 @@ static void dump_lvds_data(const struct bdb_block *block)
 	}
 }
 
-static void dump_driver_feature(const struct bdb_block *block)
+static void dump_driver_feature(const struct bdb_header *bdb,
+				const struct bdb_block *block)
 {
-	struct bdb_driver_feature *feature = block->data;
+	const struct bdb_driver_feature *feature = block->data;
 
 	printf("\tBoot Device Algorithm: %s\n", feature->boot_dev_algorithm ?
 	       "driver default" : "os default");
@@ -608,9 +621,10 @@ static void dump_driver_feature(const struct bdb_block *block)
 	       feature->legacy_crt_max_refresh);
 }
 
-static void dump_edp(const struct bdb_block *block)
+static void dump_edp(const struct bdb_header *bdb,
+		     const struct bdb_block *block)
 {
-	struct bdb_edp *edp = block->data;
+	const struct bdb_edp *edp = block->data;
 	int bpp, msa;
 	int i;
 
@@ -709,7 +723,7 @@ static void dump_edp(const struct bdb_block *block)
 }
 
 static void
-print_detail_timing_data(struct lvds_dvo_timing2 *dvo_timing)
+print_detail_timing_data(const struct lvds_dvo_timing2 *dvo_timing)
 {
 	int display, sync_start, sync_end, total;
 
@@ -737,9 +751,10 @@ print_detail_timing_data(struct lvds_dvo_timing2 *dvo_timing)
 	printf("\tclock: %d\n", dvo_timing->clock * 10);
 }
 
-static void dump_sdvo_panel_dtds(const struct bdb_block *block)
+static void dump_sdvo_panel_dtds(const struct bdb_header *bdb,
+				 const struct bdb_block *block)
 {
-	struct lvds_dvo_timing2 *dvo_timing = block->data;
+	const struct lvds_dvo_timing2 *dvo_timing = block->data;
 	int n, count;
 
 	count = block->size / sizeof(struct lvds_dvo_timing2);
@@ -749,9 +764,10 @@ static void dump_sdvo_panel_dtds(const struct bdb_block *block)
 	}
 }
 
-static void dump_sdvo_lvds_options(const struct bdb_block *block)
+static void dump_sdvo_lvds_options(const struct bdb_header *bdb,
+				   const struct bdb_block *block)
 {
-	struct bdb_sdvo_lvds_options *options = block->data;
+	const struct bdb_sdvo_lvds_options *options = block->data;
 
 	printf("\tbacklight: %d\n", options->panel_backlight);
 	printf("\th40 type: %d\n", options->h40_set_panel_type);
@@ -770,11 +786,12 @@ static void dump_sdvo_lvds_options(const struct bdb_block *block)
 	printf("\tmisc[3]: %x\n", options->panel_misc_bits_4);
 }
 
-static void dump_mipi_config(const struct bdb_block *block)
+static void dump_mipi_config(const struct bdb_header *bdb,
+			     const struct bdb_block *block)
 {
-	struct bdb_mipi_config *start = block->data;
-	struct mipi_config *config;
-	struct mipi_pps_data *pps;
+	const struct bdb_mipi_config *start = block->data;
+	const struct mipi_config *config;
+	const struct mipi_pps_data *pps;
 
 	config = &start->config[panel_type];
 	pps = &start->pps[panel_type];
@@ -852,183 +869,318 @@ static void dump_mipi_config(const struct bdb_block *block)
 
 	printf("\tMIPI PPS\n");
 	printf("\t\tPanel power ON delay: %d\n", pps->panel_on_delay);
-	printf("\t\tPanel power on to Baklight enable delay: %d\n", pps->bl_enable_delay);
+	printf("\t\tPanel power on to Backlight enable delay: %d\n", pps->bl_enable_delay);
 	printf("\t\tBacklight disable to Panel power OFF delay: %d\n", pps->bl_disable_delay);
 	printf("\t\tPanel power OFF delay: %d\n", pps->panel_off_delay);
 	printf("\t\tPanel power cycle delay: %d\n", pps->panel_power_cycle_delay);
 }
 
-static uint8_t *mipi_dump_send_packet(uint8_t *data)
+static const uint8_t *mipi_dump_send_packet(const uint8_t *data)
 {
-	uint8_t type, byte, count;
-	uint16_t len;
+	uint8_t flags, type;
+	uint16_t len, i;
 
-	byte = *data++;
-	/* get packet type and increment the pointer */
+	flags = *data++;
 	type = *data++;
-
 	len = *((uint16_t *) data);
 	data += 2;
-	printf("\t\t SEND COMMAND: ");
-	printf("0x%x 0x%x 0x%x", byte, type, len);
-	for (count = 0; count < len; count++)
-		printf(" 0x%x",*(data+count));
+
+	printf("\t\tSend DCS: Port %s, VC %d, %s, Type %02x, Length %u, Data",
+	       (flags >> 3) & 1 ? "C" : "A",
+	       (flags >> 1) & 3,
+	       flags & 1 ? "HS" : "LP",
+	       type,
+	       len);
+	for (i = 0; i < len; i++)
+		printf(" %02x", *data++);
 	printf("\n");
-	data += len;
+
 	return data;
 }
 
-static uint8_t *mipi_dump_delay(uint8_t *data)
+static const uint8_t *mipi_dump_delay(const uint8_t *data)
 {
-	printf("\t\t Delay : 0x%x 0x%x 0x%x 0x%x\n", data[0], data[1], data[2], data[3]);
-	data += 4;
-	return data;
+	printf("\t\tDelay: %u us\n", *((const uint32_t *)data));
+
+	return data + 4;
 }
 
-static uint8_t *mipi_dump_gpio(uint8_t *data)
+static const uint8_t *mipi_dump_gpio(const uint8_t *data)
 {
-	uint8_t gpio, action;
+	uint8_t index, flags;
 
-	printf("\t\t GPIO value:");
-	gpio = *data++;
+	index = *data++;
+	flags = *data++;
+
+	printf("\t\tGPIO index %u, source %d, set %d\n",
+	       index,
+	       (flags >> 1) & 3,
+	       flags & 1);
 
-	/* pull up/down */
-	action = *data++;
-	printf(" 0x%x 0x%x\n", gpio, action);
 	return data;
 }
 
-typedef uint8_t * (*fn_mipi_elem_dump)(uint8_t *data);
+static const uint8_t *mipi_dump_i2c(const uint8_t *data)
+{
+	uint8_t flags, index, bus, offset, len, i;
+	uint16_t address;
+
+	flags = *data++;
+	index = *data++;
+	bus = *data++;
+	address = *((uint16_t *) data);
+	data += 2;
+	offset = *data++;
+	len = *data++;
+
+	printf("\t\tSend I2C: Flags %02x, Index %02x, Bus %02x, Address %04x, Offset %02x, Length %u, Data",
+	       flags, index, bus, address, offset, len);
+	for (i = 0; i < len; i++)
+		printf(" %02x", *data++);
+	printf("\n");
+
+	return data;
+}
+
+typedef const uint8_t * (*fn_mipi_elem_dump)(const uint8_t *data);
 
 static const fn_mipi_elem_dump dump_elem[] = {
-	NULL, /* reserved */
-	mipi_dump_send_packet,
-	mipi_dump_delay,
-	mipi_dump_gpio,
-	NULL, /* status read; later */
+	[MIPI_SEQ_ELEM_SEND_PKT] = mipi_dump_send_packet,
+	[MIPI_SEQ_ELEM_DELAY] = mipi_dump_delay,
+	[MIPI_SEQ_ELEM_GPIO] = mipi_dump_gpio,
+	[MIPI_SEQ_ELEM_I2C] = mipi_dump_i2c,
 };
 
-static void dump_sequence(uint8_t *sequence)
+static const char * const seq_name[] = {
+	[MIPI_SEQ_ASSERT_RESET] = "MIPI_SEQ_ASSERT_RESET",
+	[MIPI_SEQ_INIT_OTP] = "MIPI_SEQ_INIT_OTP",
+	[MIPI_SEQ_DISPLAY_ON] = "MIPI_SEQ_DISPLAY_ON",
+	[MIPI_SEQ_DISPLAY_OFF]  = "MIPI_SEQ_DISPLAY_OFF",
+	[MIPI_SEQ_DEASSERT_RESET] = "MIPI_SEQ_DEASSERT_RESET",
+	[MIPI_SEQ_BACKLIGHT_ON] = "MIPI_SEQ_BACKLIGHT_ON",
+	[MIPI_SEQ_BACKLIGHT_OFF] = "MIPI_SEQ_BACKLIGHT_OFF",
+	[MIPI_SEQ_TEAR_ON] = "MIPI_SEQ_TEAR_ON",
+	[MIPI_SEQ_TEAR_OFF] = "MIPI_SEQ_TEAR_OFF",
+	[MIPI_SEQ_POWER_ON] = "MIPI_SEQ_POWER_ON",
+	[MIPI_SEQ_POWER_OFF] = "MIPI_SEQ_POWER_OFF",
+};
+
+static const char *sequence_name(enum mipi_seq seq_id)
 {
-	uint8_t *data = sequence;
-	fn_mipi_elem_dump mipi_elem_dump;
-	int index_no;
-
-	if (!sequence)
-		return;
-
-	printf("\tSequence Name: %s\n", seq_name[*data]);
-
-	/* go to the first element of the sequence */
-	data++;
-
-	/* parse each byte till we reach end of sequence byte - 0x00 */
-	while (1) {
-		index_no = *data;
-		mipi_elem_dump = dump_elem[index_no];
-		if (!mipi_elem_dump) {
-			printf("Error: Unsupported MIPI element, skipping sequence execution\n");
-			return;
-		}
-		/* goto element payload */
-		data++;
-
-		/* execute the element specifc rotines */
-		data = mipi_elem_dump(data);
-
-		/*
-		 * After processing the element, data should point to
-		 * next element or end of sequence
-		 * check if have we reached end of sequence
-		 */
-
-		if (*data == 0x00)
-			break;
-	}
+	if (seq_id < ARRAY_SIZE(seq_name) && seq_name[seq_id])
+		return seq_name[seq_id];
+	else
+		return "(unknown)";
 }
 
-static uint8_t *goto_next_sequence(uint8_t *data, int *size)
+static const uint8_t *dump_sequence(const uint8_t *data, uint8_t seq_version)
 {
-	uint16_t len;
-	int tmp = *size;
+	fn_mipi_elem_dump mipi_elem_dump;
 
-	if (--tmp < 0)
-		return NULL;
+	printf("\tSequence %u - %s\n", *data, sequence_name(*data));
 
-	/* goto first element */
+	/* Skip Sequence Byte. */
 	data++;
+
+	/* Skip Size of Sequence. */
+	if (seq_version >= 3)
+		data += 4;
+
 	while (1) {
-		switch (*data) {
-		case MIPI_SEQ_ELEM_SEND_PKT:
-			/*
-			 * skip by this element payload size
-			 * skip elem id, command flag and data type
-			 */
-			tmp -= 5;
-			if (tmp < 0)
-				return NULL;
+		uint8_t operation_byte = *data++;
+		uint8_t operation_size = 0;
 
-			data += 3;
-			len = *((uint16_t *)data);
-
-			tmp -= len;
-			if (tmp < 0)
-				return NULL;
-
-			/* skip by len */
-			data = data + 2 + len;
+		if (operation_byte == MIPI_SEQ_ELEM_END)
 			break;
-		case MIPI_SEQ_ELEM_DELAY:
-			/* skip by elem id, and delay is 4 bytes */
-			tmp -= 5;
-			if (tmp < 0)
-				return NULL;
 
-			data += 5;
-			break;
-		case MIPI_SEQ_ELEM_GPIO:
-			tmp -= 3;
-			if (tmp < 0)
-				return NULL;
+		if (operation_byte < ARRAY_SIZE(dump_elem))
+			mipi_elem_dump = dump_elem[operation_byte];
+		else
+			mipi_elem_dump = NULL;
 
-			data += 3;
-			break;
-		default:
-			printf("Unknown element\n");
+		/* Size of Operation. */
+		if (seq_version >= 3)
+			operation_size = *data++;
+
+		if (mipi_elem_dump) {
+			data = mipi_elem_dump(data);
+		} else if (operation_size) {
+			/* We have size, skip. */
+			data += operation_size;
+		} else {
+			/* No size, can't skip without parsing. */
+			printf("Error: Unsupported MIPI element %u\n",
+			       operation_byte);
 			return NULL;
-                }
+		}
+	}
 
-		/* end of sequence ? */
-		if (*data == 0)
-			break;
-        }
-
-	/* goto next sequence or end of block byte */
-	if (--tmp < 0)
-		return NULL;
-
-        data++;
-
-	/* update amount of data left for the sequence block to be parsed */
-	*size = tmp;
 	return data;
 }
 
-static uint16_t get_blocksize(void *p)
+/* Find the sequence block and size for the given panel. */
+static const uint8_t *
+find_panel_sequence_block(const struct bdb_mipi_sequence *sequence,
+			  uint16_t panel_id, uint32_t total, uint32_t *seq_size)
 {
-	uint16_t *block_ptr, block_size;
+	const uint8_t *data = &sequence->data[0];
+	uint8_t current_id;
+	uint32_t current_size;
+	int header_size = sequence->version >= 3 ? 5 : 3;
+	int index = 0;
+	int i;
 
-	block_ptr = (uint16_t *)((char *)p - 2);
-	block_size = *block_ptr;
-	return block_size;
+	/* skip new block size */
+	if (sequence->version >= 3)
+		data += 4;
+
+	for (i = 0; i < MAX_MIPI_CONFIGURATIONS && index < total; i++) {
+		if (index + header_size > total) {
+			fprintf(stderr, "Invalid sequence block (header)\n");
+			return NULL;
+		}
+
+		current_id = *(data + index);
+		if (sequence->version >= 3)
+			current_size = *((const uint32_t *)(data + index + 1));
+		else
+			current_size = *((const uint16_t *)(data + index + 1));
+
+		index += header_size;
+
+		if (index + current_size > total) {
+			fprintf(stderr, "Invalid sequence block\n");
+			return NULL;
+		}
+
+		if (current_id == panel_id) {
+			*seq_size = current_size;
+			return data + index;
+		}
+
+		index += current_size;
+	}
+
+	fprintf(stderr, "Sequence block detected but no valid configuration\n");
+
+	return NULL;
 }
 
-static void dump_mipi_sequence(const struct bdb_block *block)
+static int goto_next_sequence(const uint8_t *data, int index, int total)
 {
-	struct bdb_mipi_sequence *sequence = block->data;
-	uint8_t *data, *seq_data;
-	int i, panel_id, seq_size;
-	uint16_t block_size;
+	uint16_t len;
+
+	/* Skip Sequence Byte. */
+	for (index = index + 1; index < total; index += len) {
+		uint8_t operation_byte = *(data + index);
+		index++;
+
+		switch (operation_byte) {
+		case MIPI_SEQ_ELEM_END:
+			return index;
+		case MIPI_SEQ_ELEM_SEND_PKT:
+			if (index + 4 > total)
+				return 0;
+
+			len = *((const uint16_t *)(data + index + 2)) + 4;
+			break;
+		case MIPI_SEQ_ELEM_DELAY:
+			len = 4;
+			break;
+		case MIPI_SEQ_ELEM_GPIO:
+			len = 2;
+			break;
+		case MIPI_SEQ_ELEM_I2C:
+			if (index + 7 > total)
+				return 0;
+			len = *(data + index + 6) + 7;
+			break;
+		default:
+			fprintf(stderr, "Unknown operation byte\n");
+			return 0;
+		}
+	}
+
+	return 0;
+}
+
+static int goto_next_sequence_v3(const uint8_t *data, int index, int total)
+{
+	int seq_end;
+	uint16_t len;
+	uint32_t size_of_sequence;
+
+	/*
+	 * Could skip sequence based on Size of Sequence alone, but also do some
+	 * checking on the structure.
+	 */
+	if (total < 5) {
+		fprintf(stderr, "Too small sequence size\n");
+		return 0;
+	}
+
+	/* Skip Sequence Byte. */
+	index++;
+
+	/*
+	 * Size of Sequence. Excludes the Sequence Byte and the size itself,
+	 * includes MIPI_SEQ_ELEM_END byte, excludes the final MIPI_SEQ_END
+	 * byte.
+	 */
+	size_of_sequence = *((const uint32_t *)(data + index));
+	index += 4;
+
+	seq_end = index + size_of_sequence;
+	if (seq_end > total) {
+		fprintf(stderr, "Invalid sequence size\n");
+		return 0;
+	}
+
+	for (; index < total; index += len) {
+		uint8_t operation_byte = *(data + index);
+		index++;
+
+		if (operation_byte == MIPI_SEQ_ELEM_END) {
+			if (index != seq_end) {
+				fprintf(stderr, "Invalid element structure\n");
+				return 0;
+			}
+			return index;
+		}
+
+		len = *(data + index);
+		index++;
+
+		/*
+		 * FIXME: Would be nice to check elements like for v1/v2 in
+		 * goto_next_sequence() above.
+		 */
+		switch (operation_byte) {
+		case MIPI_SEQ_ELEM_SEND_PKT:
+		case MIPI_SEQ_ELEM_DELAY:
+		case MIPI_SEQ_ELEM_GPIO:
+		case MIPI_SEQ_ELEM_I2C:
+		case MIPI_SEQ_ELEM_SPI:
+		case MIPI_SEQ_ELEM_PMIC:
+			break;
+		default:
+			fprintf(stderr, "Unknown operation byte %u\n",
+				operation_byte);
+			break;
+		}
+	}
+
+	return 0;
+}
+
+static void dump_mipi_sequence(const struct bdb_header *bdb,
+			       const struct bdb_block *block)
+{
+	const struct bdb_mipi_sequence *sequence = block->data;
+	const uint8_t *data;
+	uint32_t seq_size;
+	int index = 0, i;
+	const uint8_t *sequence_ptrs[MIPI_SEQ_MAX] = {};
 
 	/* Check if we have sequence block as well */
 	if (!sequence) {
@@ -1036,85 +1188,49 @@ static void dump_mipi_sequence(const struct bdb_block *block)
 		return;
 	}
 
-	block_size = get_blocksize(sequence);
+	printf("\tSequence block version v%u\n", sequence->version);
 
-	/*
-	 * parse the sequence block for individual sequences
-	 */
-	seq_data = &sequence->data[0];
+	/* Fail gracefully for forward incompatible sequence block. */
+	if (sequence->version >= 4) {
+		fprintf(stderr, "Unable to parse MIPI Sequence Block v%u\n",
+			sequence->version);
+		return;
+	}
 
-	/*
-	 * sequence block is variable length and hence we need to parse and
-	 * get the sequence data for specific panel id
-	 */
-	for (i = 0; i < MAX_MIPI_CONFIGURATIONS; i++) {
-		panel_id = *seq_data;
-		seq_size = *((uint16_t *) (seq_data + 1));
-		if (panel_id == panel_type)
+	data = find_panel_sequence_block(sequence, panel_type,
+					 block->size, &seq_size);
+	if (!data)
+		return;
+
+	/* Parse the sequences. Corresponds to VBT parsing in the kernel. */
+	for (;;) {
+		uint8_t seq_id = *(data + index);
+		if (seq_id == MIPI_SEQ_END)
 			break;
 
-		/* skip the sequence including seq header of 3 bytes */
-		seq_data = seq_data + 3 + seq_size;
-		if ((seq_data - &sequence->data[0]) > block_size) {
-			printf("Sequence start is beyond sequence block size, corrupted sequence block\n");
+		if (seq_id >= MIPI_SEQ_MAX) {
+			fprintf(stderr, "Unknown sequence %u\n", seq_id);
+			return;
+		}
+
+		sequence_ptrs[seq_id] = data + index;
+
+		if (sequence->version >= 3)
+			index = goto_next_sequence_v3(data, index, seq_size);
+		else
+			index = goto_next_sequence(data, index, seq_size);
+		if (!index) {
+			fprintf(stderr, "Invalid sequence %u\n", seq_id);
 			return;
 		}
 	}
 
-	if (i == MAX_MIPI_CONFIGURATIONS) {
-		printf("Sequence block detected but no valid configuration\n");
-		return;
-	}
-
-	/* check if found sequence is completely within the sequence block
-	 * just being paranoid */
-	if (seq_size > block_size) {
-		printf("Corrupted sequence/size, bailing out\n");
-		return;
-        }
-
-	/* skip the panel id(1 byte) and seq size(2 bytes) */
-	data = (uint8_t *) calloc(1, seq_size);
-	if (data)
-		memmove(data, seq_data + 3, seq_size);
-	else {
-		printf("Memory not allocated for sequence data\n");
-		return;
-	}
-	/*
-	 * loop into the sequence data and split into multiple sequneces
-	 * There are only 5 types of sequences as of now
-	 */
-
-	/* two consecutive 0x00 indicate end of all sequences */
-        while (1) {
-		int seq_id = *data;
-		if (MIPI_SEQ_MAX > seq_id && seq_id > MIPI_SEQ_UNDEFINED)
-			dump_sequence(data);
-		 else {
-			printf("Error:undefined sequence\n");
-			goto err;
-                }
-
-		/* partial parsing to skip elements */
-		data = goto_next_sequence(data, &seq_size);
-
-		if (data == NULL) {
-			printf("Sequence elements going beyond block itself. Sequence block parsing failed\n");
-			goto err;
-                }
-
-		if (*data == 0)
-			break; /* end of sequence reached */
-	}
-        return;
-
-err:
-	free(data);
-	data = NULL;
+	/* Dump the sequences. Corresponds to sequence execution in kernel. */
+	for (i = 0; i < ARRAY_SIZE(sequence_ptrs); i++)
+		if (sequence_ptrs[i])
+			dump_sequence(sequence_ptrs[i], sequence->version);
 }
 
-
 static int
 get_device_id(unsigned char *bios, int size)
 {
@@ -1138,7 +1254,8 @@ get_device_id(unsigned char *bios, int size)
 struct dumper {
 	uint8_t id;
 	const char *name;
-	void (*dump)(const struct bdb_block *block);
+	void (*dump)(const struct bdb_header *bdb,
+		     const struct bdb_block *block);
 };
 
 struct dumper dumpers[] = {
@@ -1212,7 +1329,7 @@ struct dumper dumpers[] = {
 static void hex_dump(const struct bdb_block *block)
 {
 	int i;
-	uint8_t *p = block->data;
+	const uint8_t *p = block->data;
 
 	for (i = 0; i < block->size; i++) {
 		if (i % 16 == 0)
@@ -1230,7 +1347,7 @@ static void hex_dump(const struct bdb_block *block)
 	printf("\n\n");
 }
 
-static void dump_section(int section_id, int size)
+static void dump_section(const struct bdb_header *bdb, int section_id, int size)
 {
 	struct dumper *dumper = NULL;
 	const struct bdb_block *block;
@@ -1241,7 +1358,7 @@ static void dump_section(int section_id, int size)
 		return;
 	done[section_id] = 1;
 
-	block = find_section(section_id, size);
+	block = find_section(bdb, section_id, size);
 	if (!block)
 		return;
 
@@ -1259,7 +1376,7 @@ static void dump_section(int section_id, int size)
 
 	hex_dump(block);
 	if (dumper && dumper->dump)
-		dumper->dump(block);
+		dumper->dump(bdb, block);
 	printf("\n");
 }
 
@@ -1272,6 +1389,7 @@ int main(int argc, char **argv)
 	struct stat finfo;
 	int size;
 	struct bdb_block *block;
+	struct bdb_header *bdb;
 	char signature[17];
 	char *devid_string;
 
@@ -1354,7 +1472,7 @@ int main(int argc, char **argv)
 	printf("Available sections: ");
 
 	for (i = 0; i < 256; i++) {
-		block = find_section(i, size);
+		block = find_section(bdb, i, size);
 		if (!block)
 			continue;
 		printf("%d ", i);
@@ -1367,24 +1485,24 @@ int main(int argc, char **argv)
 	if (devid == -1)
 	    printf("Warning: could not find PCI device ID!\n");
 
-	dump_section(BDB_GENERAL_FEATURES, size);
-	dump_section(BDB_GENERAL_DEFINITIONS, size);
-	dump_section(BDB_CHILD_DEVICE_TABLE, size);
-	dump_section(BDB_LVDS_OPTIONS, size);
-	dump_section(BDB_LVDS_LFP_DATA_PTRS, size);
-	dump_section(BDB_LVDS_LFP_DATA, size);
-	dump_section(BDB_LVDS_BACKLIGHT, size);
+	dump_section(bdb, BDB_GENERAL_FEATURES, size);
+	dump_section(bdb, BDB_GENERAL_DEFINITIONS, size);
+	dump_section(bdb, BDB_CHILD_DEVICE_TABLE, size);
+	dump_section(bdb, BDB_LVDS_OPTIONS, size);
+	dump_section(bdb, BDB_LVDS_LFP_DATA_PTRS, size);
+	dump_section(bdb, BDB_LVDS_LFP_DATA, size);
+	dump_section(bdb, BDB_LVDS_BACKLIGHT, size);
 
-	dump_section(BDB_SDVO_LVDS_OPTIONS, size);
-	dump_section(BDB_SDVO_PANEL_DTDS, size);
+	dump_section(bdb, BDB_SDVO_LVDS_OPTIONS, size);
+	dump_section(bdb, BDB_SDVO_PANEL_DTDS, size);
 
-	dump_section(BDB_DRIVER_FEATURES, size);
-	dump_section(BDB_EDP, size);
-	dump_section(BDB_MIPI_CONFIG, size);
-	dump_section(BDB_MIPI_SEQUENCE, size);
+	dump_section(bdb, BDB_DRIVER_FEATURES, size);
+	dump_section(bdb, BDB_EDP, size);
+	dump_section(bdb, BDB_MIPI_CONFIG, size);
+	dump_section(bdb, BDB_MIPI_SEQUENCE, size);
 
 	for (i = 0; i < 256; i++)
-		dump_section(i, size);
+		dump_section(bdb, i, size);
 
 	return 0;
 }
