@@ -51,6 +51,7 @@
 #include <err.h>
 #include <assert.h>
 #include <intel_bufmgr.h>
+#include <zlib.h>
 
 #include "intel_chipset.h"
 #include "intel_io.h"
@@ -97,7 +98,8 @@ print_instdone(uint32_t devid, unsigned int instdone, unsigned int instdone1)
 	static int once;
 
 	if (!once) {
-		init_instdone_definitions(devid);
+		if (!init_instdone_definitions(devid))
+			return;
 		once = 1;
 	}
 
@@ -432,45 +434,131 @@ print_fault_data(unsigned devid, uint32_t data1, uint32_t data0)
 }
 
 #define MAX_RINGS 10 /* I really hope this never... */
-uint32_t head[MAX_RINGS];
-int head_ndx = 0;
-int num_rings = 0;
-static void print_batch(int is_batch, const char *ring_name, uint32_t gtt_offset)
-{
-	const char *buffer_type[2] = {  "ringbuffer", "batchbuffer" };
-	if (is_batch || !num_rings)
-		printf("%s (%s) at 0x%08x\n", buffer_type[is_batch], ring_name, gtt_offset);
-	else
-		printf("%s (%s) at 0x%08x; HEAD points to: 0x%08x\n", buffer_type[is_batch], ring_name, gtt_offset, head[head_ndx++ % num_rings] + gtt_offset);
-}
 
-static void decode(struct drm_intel_decode *ctx, bool is_batch,
-		   const char *ring_name, uint32_t gtt_offset, uint32_t *data,
+static void decode(struct drm_intel_decode *ctx,
+		   const char *buffer_name,
+		   const char *ring_name,
+		   uint64_t gtt_offset,
+		   uint32_t head_offset,
+		   uint32_t *data,
 		   int *count)
 {
 	if (!*count)
 		return;
 
-	print_batch(is_batch, ring_name, gtt_offset);
+	printf("%s (%s) at 0x%08x_%08x", buffer_name, ring_name,
+	       (unsigned)(gtt_offset >> 32),
+	       (unsigned)(gtt_offset & 0xffffffff));
+	if (head_offset != -1)
+		printf("; HEAD points to: 0x%08x_%08x",
+		       (unsigned)((head_offset + gtt_offset) >> 32),
+		       (unsigned)((head_offset + gtt_offset) & 0xffffffff));
+	printf("\n");
+
 	drm_intel_decode_set_batch_pointer(ctx, data, gtt_offset, *count);
 	drm_intel_decode(ctx);
 	*count = 0;
 }
 
+static int zlib_inflate(uint32_t **ptr, int len)
+{
+	struct z_stream_s zstream;
+	void *out;
+
+	memset(&zstream, 0, sizeof(zstream));
+
+	zstream.next_in = (unsigned char *)*ptr;
+	zstream.avail_in = 4*len;
+
+	if (inflateInit(&zstream) != Z_OK)
+		return 0;
+
+	out = malloc(128*4096); /* approximate obj size */
+	zstream.next_out = out;
+	zstream.avail_out = 40*len;
+
+	do {
+		switch (inflate(&zstream, Z_SYNC_FLUSH)) {
+		case Z_STREAM_END:
+			goto end;
+		case Z_OK:
+			break;
+		default:
+			inflateEnd(&zstream);
+			return 0;
+		}
+
+		if (zstream.avail_out)
+			break;
+
+		out = realloc(out, 2*zstream.total_out);
+		if (out == NULL) {
+			inflateEnd(&zstream);
+			return 0;
+		}
+
+		zstream.next_out = (unsigned char *)out + zstream.total_out;
+		zstream.avail_out = zstream.total_out;
+	} while (1);
+end:
+	inflateEnd(&zstream);
+	free(*ptr);
+	*ptr = out;
+	return zstream.total_out / 4;
+}
+
+static int ascii85_decode(const char *in, uint32_t **out)
+{
+	int len = 0, size = 1024;
+
+	*out = realloc(*out, sizeof(uint32_t)*size);
+	if (*out == NULL)
+		return 0;
+
+	while (*in >= '!' && *in <= 'z') {
+		uint32_t v = 0;
+
+		if (len == size) {
+			size *= 2;
+			*out = realloc(*out, sizeof(uint32_t)*size);
+			if (*out == NULL)
+				return 0;
+		}
+
+		if (*in == 'z') {
+			in++;
+		} else {
+			v += in[0] - 33; v *= 85;
+			v += in[1] - 33; v *= 85;
+			v += in[2] - 33; v *= 85;
+			v += in[3] - 33; v *= 85;
+			v += in[4] - 33;
+			in += 5;
+		}
+		(*out)[len++] = v;
+	}
+
+	return zlib_inflate(out, len);
+}
+
 static void
 read_data_file(FILE *file)
 {
 	struct drm_intel_decode *decode_ctx = NULL;
 	uint32_t devid = PCI_CHIP_I855_GM;
 	uint32_t *data = NULL;
+	uint32_t head[MAX_RINGS];
+	int head_idx = 0;
+	int num_rings = 0;
 	long long unsigned fence;
 	int data_size = 0, count = 0, line_number = 0, matched;
 	char *line = NULL;
 	size_t line_size;
 	uint32_t offset, value, ring_length = 0;
-	uint32_t gtt_offset = 0, new_gtt_offset;
+	uint64_t gtt_offset = 0, new_gtt_offset;
+	uint32_t head_offset = -1;
+	const char *buffer_name = "batch buffer";
 	char *ring_name = NULL;
-	int is_batch = 1;
 
 	while (getline(&line, &line_size, file) > 0) {
 		char *dashes;
@@ -478,36 +566,89 @@ read_data_file(FILE *file)
 
 		dashes = strstr(line, "---");
 		if (dashes) {
+			uint32_t lo, hi;
 			char *new_ring_name = malloc(dashes - line);
 			strncpy(new_ring_name, line, dashes - line);
 			new_ring_name[dashes - line - 1] = '\0';
 
-			if (num_rings == -1)
-				num_rings = head_ndx;
+			matched = sscanf(dashes, "--- gtt_offset = 0x%08x %08x\n",
+					 &hi, &lo);
+			if (matched > 0) {
+				new_gtt_offset = hi;
+				if (matched == 2) {
+					new_gtt_offset <<= 32;
+					new_gtt_offset |= lo;
+				}
 
-			matched = sscanf(dashes, "--- gtt_offset = 0x%08x\n",
-					&new_gtt_offset);
-			if (matched == 1) {
-				decode(decode_ctx, is_batch, ring_name,
-				       gtt_offset, data, &count);
+				decode(decode_ctx,
+				       buffer_name, ring_name,
+				       gtt_offset, head_offset,
+				       data, &count);
 				gtt_offset = new_gtt_offset;
-				is_batch = 1;
+				head_offset = -1;
 				free(ring_name);
 				ring_name = new_ring_name;
+				buffer_name = "batch buffer";
 				continue;
 			}
 
-			matched = sscanf(dashes, "--- ringbuffer = 0x%08x\n",
-					&new_gtt_offset);
-			if (matched == 1) {
-				decode(decode_ctx, is_batch, ring_name,
-				       gtt_offset, data, &count);
+			matched = sscanf(dashes, "--- ringbuffer = 0x%08x %08x\n",
+					 &hi, &lo);
+			if (matched > 0) {
+				new_gtt_offset = hi;
+				if (matched == 2) {
+					new_gtt_offset <<= 32;
+					new_gtt_offset |= lo;
+				}
+
+				decode(decode_ctx,
+				       buffer_name, ring_name,
+				       gtt_offset, head_offset,
+				       data, &count);
 				gtt_offset = new_gtt_offset;
-				is_batch = 0;
+				if (head_idx < num_rings)
+					head_offset = head[head_idx++];
+				else
+					head_offset = -1;
 				free(ring_name);
 				ring_name = new_ring_name;
+				buffer_name = "ring buffer";
 				continue;
 			}
+
+			matched = sscanf(dashes, "--- HW Context = 0x%08x %08x\n",
+					 &hi, &lo);
+			if (matched > 0) {
+				new_gtt_offset = hi;
+				if (matched == 2) {
+					new_gtt_offset <<= 32;
+					new_gtt_offset |= lo;
+				}
+
+				decode(decode_ctx,
+				       buffer_name, ring_name,
+				       gtt_offset, head_offset,
+				       data, &count);
+				gtt_offset = new_gtt_offset;
+				head_offset = -1;
+				free(ring_name);
+				ring_name = new_ring_name;
+				buffer_name = "HW Context";
+				continue;
+			}
+		}
+
+		if (line[0] == ':') {
+			count = ascii85_decode(line+1, &data);
+			if (count == 0) {
+				fprintf(stderr, "ASCII85 decode failed.\n");
+				exit(1);
+			}
+			decode(decode_ctx,
+			       buffer_name, ring_name,
+			       gtt_offset, head_offset,
+			       data, &count);
+			continue;
 		}
 
 		matched = sscanf(line, "%08x : %08x", &offset, &value);
@@ -515,7 +656,9 @@ read_data_file(FILE *file)
 			unsigned int reg, reg2;
 
 			/* display reg section is after the ringbuffers, don't mix them */
-			decode(decode_ctx, is_batch, ring_name, gtt_offset,
+			decode(decode_ctx,
+			       buffer_name, ring_name,
+			       gtt_offset, head_offset,
 			       data, &count);
 
 			printf("%s", line);
@@ -596,7 +739,10 @@ read_data_file(FILE *file)
 		data[count-1] = value;
 	}
 
-	decode(decode_ctx, is_batch, ring_name, gtt_offset, data, &count);
+	decode(decode_ctx,
+	       buffer_name, ring_name,
+	       gtt_offset, head_offset,
+	       data, &count);
 
 	free(data);
 	free(line);
