@@ -37,9 +37,10 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
 #include "intel_reg.h"
 #include "intel_bios.h"
+#include "intel_chipset.h"
 
 enum lid_status {
 	LID_UNKNOWN = -1,
@@ -117,7 +118,7 @@ int main(int argc, char **argv)
 {
 	int swf14, acpi_lid;
 
-	intel_get_mmio(intel_get_pci_device());
+	intel_mmio_use_pci_bar(intel_get_pci_device());
 
 	while (1) {
 		swf14 = INREG(SWF14);
